# -*- coding: utf-8 -*-
# ********************************************************************
#
#  $Id: svn_id $
#
#  High-level API for YRealTimeClock
#
#  - - - - - - - - - License information: - - - - - - - - -
#
#  Copyright (C) 2011 and beyond by Yoctopuce Sarl, Switzerland.
#
#  Yoctopuce Sarl (hereafter Licensor) grants to you a perpetual
#  non-exclusive license to use, modify, copy and integrate this
#  file into your software for the sole purpose of interfacing
#  with Yoctopuce products.
#
#  You may reproduce and distribute copies of this file in
#  source or object form, as long as the sole purpose of this
#  code is to interface with Yoctopuce products. You must retain
#  this notice in the distributed source file.
#
#  You should refer to Yoctopuce General Terms and Conditions
#  for additional information regarding your rights and
#  obligations.
#
#  THE SOFTWARE AND DOCUMENTATION ARE PROVIDED 'AS IS' WITHOUT
#  WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING
#  WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, FITNESS
#  FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO
#  EVENT SHALL LICENSOR BE LIABLE FOR ANY INCIDENTAL, SPECIAL,
#  INDIRECT OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA,
#  COST OF PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR
#  SERVICES, ANY CLAIMS BY THIRD PARTIES (INCLUDING BUT NOT
#  LIMITED TO ANY DEFENSE THEREOF), ANY CLAIMS FOR INDEMNITY OR
#  CONTRIBUTION, OR OTHER SIMILAR COSTS, WHETHER ASSERTED ON THE
#  BASIS OF CONTRACT, TORT (INCLUDING NEGLIGENCE), BREACH OF
#  WARRANTY, OR OTHERWISE.
#
# *********************************************************************
"""
Yoctopuce library: High-level API for YRealTimeClock
version: PATCH_WITH_VERSION
requires: yocto_realtimeclock_aio
requires: yocto_api
"""
from __future__ import annotations

import sys

# On MicroPython, code below will be wiped out at compile time
if sys.implementation.name != "micropython":
    # In CPython, enable edit-time type checking, including Final declaration
    from typing import Union, Final
    from collections.abc import Callable, Awaitable
    from .yocto_api import _IS_MICROPYTHON, _DYNAMIC_HELPERS
else:
    # In our micropython VM, common generic types are global built-ins
    # Others such as TypeVar should be avoided when using micropython,
    # as they produce overhead in runtime code
    # Final is translated into const() expressions before compilation
    _IS_MICROPYTHON: Final[bool] = True # noqa
    _DYNAMIC_HELPERS: Final[bool] = True # noqa

from .yocto_realtimeclock_aio import YRealTimeClock as YRealTimeClock_aio
from .yocto_api import (
    YAPIContext, YAPI, YFunction
)

# --- (YRealTimeClock class start)
if not _IS_MICROPYTHON:
    # For CPython, use strongly typed callback types
    try:
        YRealTimeClockValueCallback = Union[Callable[['YRealTimeClock', str], Awaitable[None]], None]
    except TypeError:
        YRealTimeClockValueCallback = Union[Callable, Awaitable]

# noinspection PyProtectedMember
class YRealTimeClock(YFunction):
    """
    The YRealTimeClock class provide access to the embedded real-time clock available on some Yoctopuce
    devices. It can provide current date and time, even after a power outage
    lasting several days. It is the base for automated wake-up functions provided by the WakeUpScheduler.
    The current time may represent a local time as well as an UTC time, but no automatic time change
    will occur to account for daylight saving time.

    """
    _aio: YRealTimeClock_aio
    # --- (end of YRealTimeClock class start)
    if not _IS_MICROPYTHON:
        # --- (YRealTimeClock return codes)
        UNIXTIME_INVALID: Final[int] = YAPI.INVALID_LONG
        DATETIME_INVALID: Final[str] = YAPI.INVALID_STRING
        UTCOFFSET_INVALID: Final[int] = YAPI.INVALID_INT
        TIMESET_FALSE: Final[int] = 0
        TIMESET_TRUE: Final[int] = 1
        TIMESET_INVALID: Final[int] = -1
        DISABLEHOSTSYNC_FALSE: Final[int] = 0
        DISABLEHOSTSYNC_TRUE: Final[int] = 1
        DISABLEHOSTSYNC_INVALID: Final[int] = -1
        # --- (end of YRealTimeClock return codes)


    # --- (YRealTimeClock implementation)

    @classmethod
    def FirstRealTimeClock(cls) -> Union[YRealTimeClock, None]:
        """
        Starts the enumeration of real-time clocks currently accessible.
        Use the method YRealTimeClock.nextRealTimeClock() to iterate on
        next real-time clocks.

        @return a pointer to a YRealTimeClock object, corresponding to
                the first real-time clock currently online, or a None pointer
                if there are none.
        """
        return cls._proxy(cls, YRealTimeClock_aio.FirstRealTimeClock())

    @classmethod
    def FirstRealTimeClockInContext(cls, yctx: YAPIContext) -> Union[YRealTimeClock, None]:
        """
        Starts the enumeration of real-time clocks currently accessible.
        Use the method YRealTimeClock.nextRealTimeClock() to iterate on
        next real-time clocks.

        @param yctx : a YAPI context.

        @return a pointer to a YRealTimeClock object, corresponding to
                the first real-time clock currently online, or a None pointer
                if there are none.
        """
        return cls._proxy(cls, YRealTimeClock_aio.FirstRealTimeClockInContext(yctx))

    def nextRealTimeClock(self):
        """
        Continues the enumeration of real-time clocks started using yFirstRealTimeClock().
        Caution: You can't make any assumption about the returned real-time clocks order.
        If you want to find a specific a real-time clock, use RealTimeClock.findRealTimeClock()
        and a hardwareID or a logical name.

        @return a pointer to a YRealTimeClock object, corresponding to
                a real-time clock currently online, or a None pointer
                if there are no more real-time clocks to enumerate.
        """
        return self._proxy(type(self), self._aio.nextRealTimeClock())

    if not _DYNAMIC_HELPERS:
        def get_unixTime(self) -> int:
            """
            Returns the current time in Unix format (number of elapsed seconds since Jan 1st, 1970).

            @return an integer corresponding to the current time in Unix format (number of elapsed seconds
            since Jan 1st, 1970)

            On failure, throws an exception or returns YRealTimeClock.UNIXTIME_INVALID.
            """
            return self._run(self._aio.get_unixTime())

    if not _DYNAMIC_HELPERS:
        def set_unixTime(self, newval: int) -> int:
            """
            Changes the current time. Time is specifid in Unix format (number of elapsed seconds since Jan 1st, 1970).

            @param newval : an integer corresponding to the current time

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_unixTime(newval))

    if not _DYNAMIC_HELPERS:
        def get_dateTime(self) -> str:
            """
            Returns the current time in the form "YYYY/MM/DD hh:mm:ss".

            @return a string corresponding to the current time in the form "YYYY/MM/DD hh:mm:ss"

            On failure, throws an exception or returns YRealTimeClock.DATETIME_INVALID.
            """
            return self._run(self._aio.get_dateTime())

    if not _DYNAMIC_HELPERS:
        def get_utcOffset(self) -> int:
            """
            Returns the number of seconds between current time and UTC time (time zone).

            @return an integer corresponding to the number of seconds between current time and UTC time (time zone)

            On failure, throws an exception or returns YRealTimeClock.UTCOFFSET_INVALID.
            """
            return self._run(self._aio.get_utcOffset())

    if not _DYNAMIC_HELPERS:
        def set_utcOffset(self, newval: int) -> int:
            """
            Changes the number of seconds between current time and UTC time (time zone).
            The timezone is automatically rounded to the nearest multiple of 15 minutes.
            Remember to call the saveToFlash() method of the module if the
            modification must be kept.

            @param newval : an integer corresponding to the number of seconds between current time and UTC time (time zone)

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_utcOffset(newval))

    if not _DYNAMIC_HELPERS:
        def get_timeSet(self) -> int:
            """
            Returns true if the clock has been set, and false otherwise.

            @return either YRealTimeClock.TIMESET_FALSE or YRealTimeClock.TIMESET_TRUE, according to true if
            the clock has been set, and false otherwise

            On failure, throws an exception or returns YRealTimeClock.TIMESET_INVALID.
            """
            return self._run(self._aio.get_timeSet())

    if not _DYNAMIC_HELPERS:
        def get_disableHostSync(self) -> int:
            """
            Returns true if the automatic clock synchronization with host has been disabled,
            and false otherwise.

            @return either YRealTimeClock.DISABLEHOSTSYNC_FALSE or YRealTimeClock.DISABLEHOSTSYNC_TRUE,
            according to true if the automatic clock synchronization with host has been disabled,
                    and false otherwise

            On failure, throws an exception or returns YRealTimeClock.DISABLEHOSTSYNC_INVALID.
            """
            return self._run(self._aio.get_disableHostSync())

    if not _DYNAMIC_HELPERS:
        def set_disableHostSync(self, newval: int) -> int:
            """
            Changes the automatic clock synchronization with host working state.
            To disable automatic synchronization, set the value to true.
            To enable automatic synchronization (default), set the value to false.

            If you want the change to be kept after a device reboot,
            make sure  to call the matching module saveToFlash().

            @param newval : either YRealTimeClock.DISABLEHOSTSYNC_FALSE or YRealTimeClock.DISABLEHOSTSYNC_TRUE,
            according to the automatic clock synchronization with host working state

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_disableHostSync(newval))

    @classmethod
    def FindRealTimeClock(cls, func: str) -> YRealTimeClock:
        """
        Retrieves a real-time clock for a given identifier.
        The identifier can be specified using several formats:

        - FunctionLogicalName
        - ModuleSerialNumber.FunctionIdentifier
        - ModuleSerialNumber.FunctionLogicalName
        - ModuleLogicalName.FunctionIdentifier
        - ModuleLogicalName.FunctionLogicalName


        This function does not require that the real-time clock is online at the time
        it is invoked. The returned object is nevertheless valid.
        Use the method YRealTimeClock.isOnline() to test if the real-time clock is
        indeed online at a given time. In case of ambiguity when looking for
        a real-time clock by logical name, no error is notified: the first instance
        found is returned. The search is performed first by hardware name,
        then by logical name.

        If a call to this object's is_online() method returns FALSE although
        you are certain that the matching device is plugged, make sure that you did
        call registerHub() at application initialization time.

        @param func : a string that uniquely characterizes the real-time clock, for instance
                YHUBGSM5.realTimeClock.

        @return a YRealTimeClock object allowing you to drive the real-time clock.
        """
        return cls._proxy(cls, YRealTimeClock_aio.FindRealTimeClock(func))

    @classmethod
    def FindRealTimeClockInContext(cls, yctx: YAPIContext, func: str) -> YRealTimeClock:
        """
        Retrieves a real-time clock for a given identifier in a YAPI context.
        The identifier can be specified using several formats:

        - FunctionLogicalName
        - ModuleSerialNumber.FunctionIdentifier
        - ModuleSerialNumber.FunctionLogicalName
        - ModuleLogicalName.FunctionIdentifier
        - ModuleLogicalName.FunctionLogicalName


        This function does not require that the real-time clock is online at the time
        it is invoked. The returned object is nevertheless valid.
        Use the method YRealTimeClock.isOnline() to test if the real-time clock is
        indeed online at a given time. In case of ambiguity when looking for
        a real-time clock by logical name, no error is notified: the first instance
        found is returned. The search is performed first by hardware name,
        then by logical name.

        @param yctx : a YAPI context
        @param func : a string that uniquely characterizes the real-time clock, for instance
                YHUBGSM5.realTimeClock.

        @return a YRealTimeClock object allowing you to drive the real-time clock.
        """
        return cls._proxy(cls, YRealTimeClock_aio.FindRealTimeClockInContext(yctx, func))

    if not _IS_MICROPYTHON:
        def registerValueCallback(self, callback: YRealTimeClockValueCallback) -> int:
            """
            Registers the callback function that is invoked on every change of advertised value.
            The callback is invoked only during the execution of ySleep or yHandleEvents.
            This provides control over the time when the callback is triggered. For good responsiveness, remember to call
            one of these two functions periodically. To unregister a callback, pass a None pointer as argument.

            @param callback : the callback function to call, or a None pointer. The callback function should take two
                    arguments: the function object of which the value has changed, and the character string describing
                    the new advertised value.
            @noreturn
            """
            return super().registerValueCallback(callback)

    # --- (end of YRealTimeClock implementation)




# -*- coding: utf-8 -*-
# ********************************************************************
#
#  $Id: svn_id $
#
#  High-level API for YMultiCellWeighScale
#
#  - - - - - - - - - License information: - - - - - - - - -
#
#  Copyright (C) 2011 and beyond by Yoctopuce Sarl, Switzerland.
#
#  Yoctopuce Sarl (hereafter Licensor) grants to you a perpetual
#  non-exclusive license to use, modify, copy and integrate this
#  file into your software for the sole purpose of interfacing
#  with Yoctopuce products.
#
#  You may reproduce and distribute copies of this file in
#  source or object form, as long as the sole purpose of this
#  code is to interface with Yoctopuce products. You must retain
#  this notice in the distributed source file.
#
#  You should refer to Yoctopuce General Terms and Conditions
#  for additional information regarding your rights and
#  obligations.
#
#  THE SOFTWARE AND DOCUMENTATION ARE PROVIDED 'AS IS' WITHOUT
#  WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING
#  WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, FITNESS
#  FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO
#  EVENT SHALL LICENSOR BE LIABLE FOR ANY INCIDENTAL, SPECIAL,
#  INDIRECT OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA,
#  COST OF PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR
#  SERVICES, ANY CLAIMS BY THIRD PARTIES (INCLUDING BUT NOT
#  LIMITED TO ANY DEFENSE THEREOF), ANY CLAIMS FOR INDEMNITY OR
#  CONTRIBUTION, OR OTHER SIMILAR COSTS, WHETHER ASSERTED ON THE
#  BASIS OF CONTRACT, TORT (INCLUDING NEGLIGENCE), BREACH OF
#  WARRANTY, OR OTHERWISE.
#
# *********************************************************************
"""
Yoctopuce library: High-level API for YMultiCellWeighScale
version: PATCH_WITH_VERSION
requires: yocto_multicellweighscale_aio
requires: yocto_api
"""
from __future__ import annotations

import sys

# On MicroPython, code below will be wiped out at compile time
if sys.implementation.name != "micropython":
    # In CPython, enable edit-time type checking, including Final declaration
    from typing import Union, Final
    from collections.abc import Callable, Awaitable
    from .yocto_api import _IS_MICROPYTHON, _DYNAMIC_HELPERS
else:
    # In our micropython VM, common generic types are global built-ins
    # Others such as TypeVar should be avoided when using micropython,
    # as they produce overhead in runtime code
    # Final is translated into const() expressions before compilation
    _IS_MICROPYTHON: Final[bool] = True # noqa
    _DYNAMIC_HELPERS: Final[bool] = True # noqa

from .yocto_multicellweighscale_aio import YMultiCellWeighScale as YMultiCellWeighScale_aio
from .yocto_api import (
    YAPIContext, YAPI, YSensor, YMeasure
)

# --- (YMultiCellWeighScale class start)
if not _IS_MICROPYTHON:
    # For CPython, use strongly typed callback types
    try:
        YMultiCellWeighScaleValueCallback = Union[Callable[['YMultiCellWeighScale', str], Awaitable[None]], None]
        YMultiCellWeighScaleTimedReportCallback = Union[Callable[['YMultiCellWeighScale', YMeasure], Awaitable[None]], None]
    except TypeError:
        YMultiCellWeighScaleValueCallback = Union[Callable, Awaitable]
        YMultiCellWeighScaleTimedReportCallback = Union[Callable, Awaitable]

# noinspection PyProtectedMember
class YMultiCellWeighScale(YSensor):
    """
    The YMultiCellWeighScale class provides a weight measurement from a set of ratiometric
    sensors. It can be used to control the bridge excitation parameters, in order to avoid
    measure shifts caused by temperature variation in the electronics, and can also
    automatically apply an additional correction factor based on temperature to
    compensate for offsets in the load cells themselves.

    """
    _aio: YMultiCellWeighScale_aio
    # --- (end of YMultiCellWeighScale class start)
    if not _IS_MICROPYTHON:
        # --- (YMultiCellWeighScale return codes)
        CELLCOUNT_INVALID: Final[int] = YAPI.INVALID_UINT
        TEMPAVGADAPTRATIO_INVALID: Final[float] = YAPI.INVALID_DOUBLE
        TEMPCHGADAPTRATIO_INVALID: Final[float] = YAPI.INVALID_DOUBLE
        COMPTEMPAVG_INVALID: Final[float] = YAPI.INVALID_DOUBLE
        COMPTEMPCHG_INVALID: Final[float] = YAPI.INVALID_DOUBLE
        COMPENSATION_INVALID: Final[float] = YAPI.INVALID_DOUBLE
        ZEROTRACKING_INVALID: Final[float] = YAPI.INVALID_DOUBLE
        COMMAND_INVALID: Final[str] = YAPI.INVALID_STRING
        EXTERNALSENSE_FALSE: Final[int] = 0
        EXTERNALSENSE_TRUE: Final[int] = 1
        EXTERNALSENSE_INVALID: Final[int] = -1
        EXCITATION_OFF: Final[int] = 0
        EXCITATION_DC: Final[int] = 1
        EXCITATION_AC: Final[int] = 2
        EXCITATION_INVALID: Final[int] = -1
        # --- (end of YMultiCellWeighScale return codes)


    # --- (YMultiCellWeighScale implementation)

    @classmethod
    def FirstMultiCellWeighScale(cls) -> Union[YMultiCellWeighScale, None]:
        """
        Starts the enumeration of multi-cell weighing scale sensors currently accessible.
        Use the method YMultiCellWeighScale.nextMultiCellWeighScale() to iterate on
        next multi-cell weighing scale sensors.

        @return a pointer to a YMultiCellWeighScale object, corresponding to
                the first multi-cell weighing scale sensor currently online, or a None pointer
                if there are none.
        """
        return cls._proxy(cls, YMultiCellWeighScale_aio.FirstMultiCellWeighScale())

    @classmethod
    def FirstMultiCellWeighScaleInContext(cls, yctx: YAPIContext) -> Union[YMultiCellWeighScale, None]:
        """
        Starts the enumeration of multi-cell weighing scale sensors currently accessible.
        Use the method YMultiCellWeighScale.nextMultiCellWeighScale() to iterate on
        next multi-cell weighing scale sensors.

        @param yctx : a YAPI context.

        @return a pointer to a YMultiCellWeighScale object, corresponding to
                the first multi-cell weighing scale sensor currently online, or a None pointer
                if there are none.
        """
        return cls._proxy(cls, YMultiCellWeighScale_aio.FirstMultiCellWeighScaleInContext(yctx))

    def nextMultiCellWeighScale(self):
        """
        Continues the enumeration of multi-cell weighing scale sensors started using yFirstMultiCellWeighScale().
        Caution: You can't make any assumption about the returned multi-cell weighing scale sensors order.
        If you want to find a specific a multi-cell weighing scale sensor, use
        MultiCellWeighScale.findMultiCellWeighScale()
        and a hardwareID or a logical name.

        @return a pointer to a YMultiCellWeighScale object, corresponding to
                a multi-cell weighing scale sensor currently online, or a None pointer
                if there are no more multi-cell weighing scale sensors to enumerate.
        """
        return self._proxy(type(self), self._aio.nextMultiCellWeighScale())

    if not _DYNAMIC_HELPERS:
        def set_unit(self, newval: str) -> int:
            """
            Changes the measuring unit for the weight.
            Remember to call the saveToFlash() method of the module if the
            modification must be kept.

            @param newval : a string corresponding to the measuring unit for the weight

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_unit(newval))

    if not _DYNAMIC_HELPERS:
        def get_cellCount(self) -> int:
            """
            Returns the number of load cells in use.

            @return an integer corresponding to the number of load cells in use

            On failure, throws an exception or returns YMultiCellWeighScale.CELLCOUNT_INVALID.
            """
            return self._run(self._aio.get_cellCount())

    if not _DYNAMIC_HELPERS:
        def set_cellCount(self, newval: int) -> int:
            """
            Changes the number of load cells in use. Remember to call the saveToFlash()
            method of the module if the modification must be kept.

            @param newval : an integer corresponding to the number of load cells in use

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_cellCount(newval))

    if not _DYNAMIC_HELPERS:
        def get_externalSense(self) -> int:
            """
            Returns true if entry 4 is used as external sense for 6-wires load cells.

            @return either YMultiCellWeighScale.EXTERNALSENSE_FALSE or YMultiCellWeighScale.EXTERNALSENSE_TRUE,
            according to true if entry 4 is used as external sense for 6-wires load cells

            On failure, throws an exception or returns YMultiCellWeighScale.EXTERNALSENSE_INVALID.
            """
            return self._run(self._aio.get_externalSense())

    if not _DYNAMIC_HELPERS:
        def set_externalSense(self, newval: int) -> int:
            """
            Changes the configuration to tell if entry 4 is used as external sense for
            6-wires load cells. Remember to call the saveToFlash() method of the
            module if the modification must be kept.

            @param newval : either YMultiCellWeighScale.EXTERNALSENSE_FALSE or
            YMultiCellWeighScale.EXTERNALSENSE_TRUE, according to the configuration to tell if entry 4 is used
            as external sense for
                    6-wires load cells

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_externalSense(newval))

    if not _DYNAMIC_HELPERS:
        def get_excitation(self) -> int:
            """
            Returns the current load cell bridge excitation method.

            @return a value among YMultiCellWeighScale.EXCITATION_OFF, YMultiCellWeighScale.EXCITATION_DC and
            YMultiCellWeighScale.EXCITATION_AC corresponding to the current load cell bridge excitation method

            On failure, throws an exception or returns YMultiCellWeighScale.EXCITATION_INVALID.
            """
            return self._run(self._aio.get_excitation())

    if not _DYNAMIC_HELPERS:
        def set_excitation(self, newval: int) -> int:
            """
            Changes the current load cell bridge excitation method.
            Remember to call the saveToFlash() method of the module if the
            modification must be kept.

            @param newval : a value among YMultiCellWeighScale.EXCITATION_OFF,
            YMultiCellWeighScale.EXCITATION_DC and YMultiCellWeighScale.EXCITATION_AC corresponding to the
            current load cell bridge excitation method

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_excitation(newval))

    if not _DYNAMIC_HELPERS:
        def set_tempAvgAdaptRatio(self, newval: float) -> int:
            """
            Changes the averaged temperature update rate, in per mille.
            The purpose of this adaptation ratio is to model the thermal inertia of the load cell.
            The averaged temperature is updated every 10 seconds, by applying this adaptation rate
            to the difference between the measures ambient temperature and the current compensation
            temperature. The standard rate is 0.2 per mille, and the maximal rate is 65 per mille.
            Remember to call the saveToFlash() method of the module if the
            modification must be kept.

            @param newval : a floating point number corresponding to the averaged temperature update rate, in per mille

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_tempAvgAdaptRatio(newval))

    if not _DYNAMIC_HELPERS:
        def get_tempAvgAdaptRatio(self) -> float:
            """
            Returns the averaged temperature update rate, in per mille.
            The purpose of this adaptation ratio is to model the thermal inertia of the load cell.
            The averaged temperature is updated every 10 seconds, by applying this adaptation rate
            to the difference between the measures ambient temperature and the current compensation
            temperature. The standard rate is 0.2 per mille, and the maximal rate is 65 per mille.

            @return a floating point number corresponding to the averaged temperature update rate, in per mille

            On failure, throws an exception or returns YMultiCellWeighScale.TEMPAVGADAPTRATIO_INVALID.
            """
            return self._run(self._aio.get_tempAvgAdaptRatio())

    if not _DYNAMIC_HELPERS:
        def set_tempChgAdaptRatio(self, newval: float) -> int:
            """
            Changes the temperature change update rate, in per mille.
            The temperature change is updated every 10 seconds, by applying this adaptation rate
            to the difference between the measures ambient temperature and the current temperature used for
            change compensation. The standard rate is 0.6 per mille, and the maximal rate is 65 per mille.
            Remember to call the saveToFlash() method of the module if the
            modification must be kept.

            @param newval : a floating point number corresponding to the temperature change update rate, in per mille

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_tempChgAdaptRatio(newval))

    if not _DYNAMIC_HELPERS:
        def get_tempChgAdaptRatio(self) -> float:
            """
            Returns the temperature change update rate, in per mille.
            The temperature change is updated every 10 seconds, by applying this adaptation rate
            to the difference between the measures ambient temperature and the current temperature used for
            change compensation. The standard rate is 0.6 per mille, and the maximal rate is 65 per mille.

            @return a floating point number corresponding to the temperature change update rate, in per mille

            On failure, throws an exception or returns YMultiCellWeighScale.TEMPCHGADAPTRATIO_INVALID.
            """
            return self._run(self._aio.get_tempChgAdaptRatio())

    if not _DYNAMIC_HELPERS:
        def get_compTempAvg(self) -> float:
            """
            Returns the current averaged temperature, used for thermal compensation.

            @return a floating point number corresponding to the current averaged temperature, used for thermal compensation

            On failure, throws an exception or returns YMultiCellWeighScale.COMPTEMPAVG_INVALID.
            """
            return self._run(self._aio.get_compTempAvg())

    if not _DYNAMIC_HELPERS:
        def get_compTempChg(self) -> float:
            """
            Returns the current temperature variation, used for thermal compensation.

            @return a floating point number corresponding to the current temperature variation, used for
            thermal compensation

            On failure, throws an exception or returns YMultiCellWeighScale.COMPTEMPCHG_INVALID.
            """
            return self._run(self._aio.get_compTempChg())

    if not _DYNAMIC_HELPERS:
        def get_compensation(self) -> float:
            """
            Returns the current current thermal compensation value.

            @return a floating point number corresponding to the current current thermal compensation value

            On failure, throws an exception or returns YMultiCellWeighScale.COMPENSATION_INVALID.
            """
            return self._run(self._aio.get_compensation())

    if not _DYNAMIC_HELPERS:
        def set_zeroTracking(self, newval: float) -> int:
            """
            Changes the zero tracking threshold value. When this threshold is larger than
            zero, any measure under the threshold will automatically be ignored and the
            zero compensation will be updated.
            Remember to call the saveToFlash() method of the module if the
            modification must be kept.

            @param newval : a floating point number corresponding to the zero tracking threshold value

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_zeroTracking(newval))

    if not _DYNAMIC_HELPERS:
        def get_zeroTracking(self) -> float:
            """
            Returns the zero tracking threshold value. When this threshold is larger than
            zero, any measure under the threshold will automatically be ignored and the
            zero compensation will be updated.

            @return a floating point number corresponding to the zero tracking threshold value

            On failure, throws an exception or returns YMultiCellWeighScale.ZEROTRACKING_INVALID.
            """
            return self._run(self._aio.get_zeroTracking())

    if not _DYNAMIC_HELPERS:
        def set_command(self, newval: str) -> int:
            return self._run(self._aio.set_command(newval))

    @classmethod
    def FindMultiCellWeighScale(cls, func: str) -> YMultiCellWeighScale:
        """
        Retrieves a multi-cell weighing scale sensor for a given identifier.
        The identifier can be specified using several formats:

        - FunctionLogicalName
        - ModuleSerialNumber.FunctionIdentifier
        - ModuleSerialNumber.FunctionLogicalName
        - ModuleLogicalName.FunctionIdentifier
        - ModuleLogicalName.FunctionLogicalName


        This function does not require that the multi-cell weighing scale sensor is online at the time
        it is invoked. The returned object is nevertheless valid.
        Use the method YMultiCellWeighScale.isOnline() to test if the multi-cell weighing scale sensor is
        indeed online at a given time. In case of ambiguity when looking for
        a multi-cell weighing scale sensor by logical name, no error is notified: the first instance
        found is returned. The search is performed first by hardware name,
        then by logical name.

        If a call to this object's is_online() method returns FALSE although
        you are certain that the matching device is plugged, make sure that you did
        call registerHub() at application initialization time.

        @param func : a string that uniquely characterizes the multi-cell weighing scale sensor, for instance
                YWMBRDG1.multiCellWeighScale.

        @return a YMultiCellWeighScale object allowing you to drive the multi-cell weighing scale sensor.
        """
        return cls._proxy(cls, YMultiCellWeighScale_aio.FindMultiCellWeighScale(func))

    @classmethod
    def FindMultiCellWeighScaleInContext(cls, yctx: YAPIContext, func: str) -> YMultiCellWeighScale:
        """
        Retrieves a multi-cell weighing scale sensor for a given identifier in a YAPI context.
        The identifier can be specified using several formats:

        - FunctionLogicalName
        - ModuleSerialNumber.FunctionIdentifier
        - ModuleSerialNumber.FunctionLogicalName
        - ModuleLogicalName.FunctionIdentifier
        - ModuleLogicalName.FunctionLogicalName


        This function does not require that the multi-cell weighing scale sensor is online at the time
        it is invoked. The returned object is nevertheless valid.
        Use the method YMultiCellWeighScale.isOnline() to test if the multi-cell weighing scale sensor is
        indeed online at a given time. In case of ambiguity when looking for
        a multi-cell weighing scale sensor by logical name, no error is notified: the first instance
        found is returned. The search is performed first by hardware name,
        then by logical name.

        @param yctx : a YAPI context
        @param func : a string that uniquely characterizes the multi-cell weighing scale sensor, for instance
                YWMBRDG1.multiCellWeighScale.

        @return a YMultiCellWeighScale object allowing you to drive the multi-cell weighing scale sensor.
        """
        return cls._proxy(cls, YMultiCellWeighScale_aio.FindMultiCellWeighScaleInContext(yctx, func))

    if not _IS_MICROPYTHON:
        def registerValueCallback(self, callback: YMultiCellWeighScaleValueCallback) -> int:
            """
            Registers the callback function that is invoked on every change of advertised value.
            The callback is invoked only during the execution of ySleep or yHandleEvents.
            This provides control over the time when the callback is triggered. For good responsiveness, remember to call
            one of these two functions periodically. To unregister a callback, pass a None pointer as argument.

            @param callback : the callback function to call, or a None pointer. The callback function should take two
                    arguments: the function object of which the value has changed, and the character string describing
                    the new advertised value.
            @noreturn
            """
            return super().registerValueCallback(callback)

    if not _IS_MICROPYTHON:
        def registerTimedReportCallback(self, callback: YMultiCellWeighScaleTimedReportCallback) -> int:
            """
            Registers the callback function that is invoked on every periodic timed notification.
            The callback is invoked only during the execution of ySleep or yHandleEvents.
            This provides control over the time when the callback is triggered. For good responsiveness, remember to call
            one of these two functions periodically. To unregister a callback, pass a None pointer as argument.

            @param callback : the callback function to call, or a None pointer. The callback function should take two
                    arguments: the function object of which the value has changed, and an YMeasure object describing
                    the new advertised value.
            @noreturn
            """
            return super().registerTimedReportCallback(callback)

    if not _DYNAMIC_HELPERS:
        def tare(self) -> int:
            """
            Adapts the load cell signal bias (stored in the corresponding genericSensor)
            so that the current signal corresponds to a zero weight. Remember to call the
            saveToFlash() method of the module if the modification must be kept.

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.tare())

    if not _DYNAMIC_HELPERS:
        def setupSpan(self, currWeight: float, maxWeight: float) -> int:
            """
            Configures the load cells span parameters (stored in the corresponding genericSensors)
            so that the current signal corresponds to the specified reference weight.

            @param currWeight : reference weight presently on the load cell.
            @param maxWeight : maximum weight to be expected on the load cell.

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.setupSpan(currWeight, maxWeight))

    # --- (end of YMultiCellWeighScale implementation)




# -*- coding: utf-8 -*-
# ********************************************************************
#
#  $Id: svn_id $
#
#  Asyncio implementation of YMicroPython
#
#  - - - - - - - - - License information: - - - - - - - - -
#
#  Copyright (C) 2011 and beyond by Yoctopuce Sarl, Switzerland.
#
#  Yoctopuce Sarl (hereafter Licensor) grants to you a perpetual
#  non-exclusive license to use, modify, copy and integrate this
#  file into your software for the sole purpose of interfacing
#  with Yoctopuce products.
#
#  You may reproduce and distribute copies of this file in
#  source or object form, as long as the sole purpose of this
#  code is to interface with Yoctopuce products. You must retain
#  this notice in the distributed source file.
#
#  You should refer to Yoctopuce General Terms and Conditions
#  for additional information regarding your rights and
#  obligations.
#
#  THE SOFTWARE AND DOCUMENTATION ARE PROVIDED 'AS IS' WITHOUT
#  WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING
#  WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, FITNESS
#  FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO
#  EVENT SHALL LICENSOR BE LIABLE FOR ANY INCIDENTAL, SPECIAL,
#  INDIRECT OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA,
#  COST OF PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR
#  SERVICES, ANY CLAIMS BY THIRD PARTIES (INCLUDING BUT NOT
#  LIMITED TO ANY DEFENSE THEREOF), ANY CLAIMS FOR INDEMNITY OR
#  CONTRIBUTION, OR OTHER SIMILAR COSTS, WHETHER ASSERTED ON THE
#  BASIS OF CONTRACT, TORT (INCLUDING NEGLIGENCE), BREACH OF
#  WARRANTY, OR OTHERWISE.
#
# *********************************************************************
"""
Yoctopuce library: Asyncio implementation of YMicroPython
version: PATCH_WITH_VERSION
requires: yocto_api_aio
"""
from __future__ import annotations

import sys

# On MicroPython, code below will be wiped out at compile time
if sys.implementation.name != "micropython":
    # In CPython, enable edit-time type checking, including Final declaration
    from typing import Any, Union, Final
    from collections.abc import Callable, Awaitable
    from .yocto_api_aio import const, _IS_MICROPYTHON
else:
    # In our micropython VM, common generic types are global built-ins
    # Others such as TypeVar should be avoided when using micropython,
    # as they produce overhead in runtime code
    # Final is translated into const() expressions before compilation
    _IS_MICROPYTHON: Final[bool] = True # noqa

from .yocto_api_aio import (
    YAPIContext, YAPI, YAPI_Exception, HwId, hwid2str, YFunction, xbytearray, xarray
)

async def yInternalEventCallback(obj: YMicroPython, value: str):
    await obj._internalEventHandler(value)

# --- (YMicroPython class start)
if not _IS_MICROPYTHON:
    # For CPython, use strongly typed callback types
    try:
        YMicroPythonValueCallback = Union[Callable[['YMicroPython', str], Awaitable[None]], None]
        YMicroPythonLogCallback = Union[Callable[['YMicroPython', str], Awaitable[None]], None]
    except TypeError:
        YMicroPythonValueCallback = Union[Callable, Awaitable]
        YMicroPythonLogCallback = Union[Callable, Awaitable]

# noinspection PyProtectedMember
class YMicroPython(YFunction):
    """
    The YMicroPython class provides control of the MicroPython interpreter
    that can be found on some Yoctopuce devices.

    """
    # --- (end of YMicroPython class start)
    if not _IS_MICROPYTHON:
        # --- (YMicroPython return codes)
        LASTMSG_INVALID: Final[str] = YAPI.INVALID_STRING
        HEAPUSAGE_INVALID: Final[int] = YAPI.INVALID_UINT
        XHEAPUSAGE_INVALID: Final[int] = YAPI.INVALID_UINT
        CURRENTSCRIPT_INVALID: Final[str] = YAPI.INVALID_STRING
        STARTUPSCRIPT_INVALID: Final[str] = YAPI.INVALID_STRING
        COMMAND_INVALID: Final[str] = YAPI.INVALID_STRING
        DEBUGMODE_OFF: Final[int] = 0
        DEBUGMODE_ON: Final[int] = 1
        DEBUGMODE_INVALID: Final[int] = -1
        # --- (end of YMicroPython return codes)

    # --- (YMicroPython attributes declaration)
    _lastMsg: str
    _heapUsage: int
    _xheapUsage: int
    _currentScript: str
    _startupScript: str
    _debugMode: int
    _command: str
    _valueCallback: YMicroPythonValueCallback
    _logCallback: YMicroPythonLogCallback
    _isFirstCb: bool
    _prevCbPos: int
    _logPos: int
    _prevPartialLog: str
    # --- (end of YMicroPython attributes declaration)


    def __init__(self, yctx: YAPIContext, func: str):
        super().__init__(yctx, func)
        self._className = 'MicroPython'
        # --- (YMicroPython constructor)
        self._lastMsg = YMicroPython.LASTMSG_INVALID
        self._heapUsage = YMicroPython.HEAPUSAGE_INVALID
        self._xheapUsage = YMicroPython.XHEAPUSAGE_INVALID
        self._currentScript = YMicroPython.CURRENTSCRIPT_INVALID
        self._startupScript = YMicroPython.STARTUPSCRIPT_INVALID
        self._debugMode = YMicroPython.DEBUGMODE_INVALID
        self._command = YMicroPython.COMMAND_INVALID
        self._isFirstCb = False
        self._prevCbPos = 0
        self._logPos = 0
        self._prevPartialLog = ''
        # --- (end of YMicroPython constructor)

    # --- (YMicroPython implementation)

    @staticmethod
    def FirstMicroPython() -> Union[YMicroPython, None]:
        """
        Starts the enumeration of MicroPython interpreters currently accessible.
        Use the method YMicroPython.nextMicroPython() to iterate on
        next MicroPython interpreters.

        @return a pointer to a YMicroPython object, corresponding to
                the first MicroPython interpreter currently online, or a None pointer
                if there are none.
        """
        next_hwid: Union[HwId, None] = YAPI._yHash.getFirstHardwareId('MicroPython')
        if not next_hwid:
            return None
        return YMicroPython.FindMicroPython(hwid2str(next_hwid))

    @staticmethod
    def FirstMicroPythonInContext(yctx: YAPIContext) -> Union[YMicroPython, None]:
        """
        Starts the enumeration of MicroPython interpreters currently accessible.
        Use the method YMicroPython.nextMicroPython() to iterate on
        next MicroPython interpreters.

        @param yctx : a YAPI context.

        @return a pointer to a YMicroPython object, corresponding to
                the first MicroPython interpreter currently online, or a None pointer
                if there are none.
        """
        next_hwid: Union[HwId, None] = yctx._yHash.getFirstHardwareId('MicroPython')
        if not next_hwid:
            return None
        return YMicroPython.FindMicroPythonInContext(yctx, hwid2str(next_hwid))

    def nextMicroPython(self):
        """
        Continues the enumeration of MicroPython interpreters started using yFirstMicroPython().
        Caution: You can't make any assumption about the returned MicroPython interpreters order.
        If you want to find a specific a MicroPython interpreter, use MicroPython.findMicroPython()
        and a hardwareID or a logical name.

        @return a pointer to a YMicroPython object, corresponding to
                a MicroPython interpreter currently online, or a None pointer
                if there are no more MicroPython interpreters to enumerate.
        """
        next_hwid: Union[HwId, None] = None
        try:
            hwid: HwId = self._yapi._yHash.resolveHwID(self._className, self._func)
            next_hwid = self._yapi._yHash.getNextHardwareId(self._className, hwid)
        except YAPI_Exception:
            pass
        if not next_hwid:
            return None
        return YMicroPython.FindMicroPythonInContext(self._yapi, hwid2str(next_hwid))

    def _parseAttr(self, json_val: dict) -> None:
        if 'lastMsg' in json_val:
            self._lastMsg = json_val["lastMsg"]
        if 'heapUsage' in json_val:
            self._heapUsage = json_val["heapUsage"]
        if 'xheapUsage' in json_val:
            self._xheapUsage = json_val["xheapUsage"]
        if 'currentScript' in json_val:
            self._currentScript = json_val["currentScript"]
        if 'startupScript' in json_val:
            self._startupScript = json_val["startupScript"]
        if 'debugMode' in json_val:
            self._debugMode = json_val["debugMode"] > 0
        if 'command' in json_val:
            self._command = json_val["command"]
        super()._parseAttr(json_val)

    async def get_lastMsg(self) -> str:
        """
        Returns the last message produced by a python script.

        @return a string corresponding to the last message produced by a python script

        On failure, throws an exception or returns YMicroPython.LASTMSG_INVALID.
        """
        res: str
        if self._cacheExpiration <= YAPI.GetTickCount():
            if await self.load(self._yapi.GetCacheValidity()) != YAPI.SUCCESS:
                return YMicroPython.LASTMSG_INVALID
        res = self._lastMsg
        return res

    async def get_heapUsage(self) -> int:
        """
        Returns the percentage of micropython main memory in use,
        as observed at the end of the last garbage collection.

        @return an integer corresponding to the percentage of micropython main memory in use,
                as observed at the end of the last garbage collection

        On failure, throws an exception or returns YMicroPython.HEAPUSAGE_INVALID.
        """
        res: int
        if self._cacheExpiration <= YAPI.GetTickCount():
            if await self.load(self._yapi.GetCacheValidity()) != YAPI.SUCCESS:
                return YMicroPython.HEAPUSAGE_INVALID
        res = self._heapUsage
        return res

    async def get_xheapUsage(self) -> int:
        """
        Returns the percentage of micropython external memory in use,
        as observed at the end of the last garbage collection.

        @return an integer corresponding to the percentage of micropython external memory in use,
                as observed at the end of the last garbage collection

        On failure, throws an exception or returns YMicroPython.XHEAPUSAGE_INVALID.
        """
        res: int
        if self._cacheExpiration <= YAPI.GetTickCount():
            if await self.load(self._yapi.GetCacheValidity()) != YAPI.SUCCESS:
                return YMicroPython.XHEAPUSAGE_INVALID
        res = self._xheapUsage
        return res

    async def get_currentScript(self) -> str:
        """
        Returns the name of currently active script, if any.

        @return a string corresponding to the name of currently active script, if any

        On failure, throws an exception or returns YMicroPython.CURRENTSCRIPT_INVALID.
        """
        res: str
        if self._cacheExpiration <= YAPI.GetTickCount():
            if await self.load(self._yapi.GetCacheValidity()) != YAPI.SUCCESS:
                return YMicroPython.CURRENTSCRIPT_INVALID
        res = self._currentScript
        return res

    async def set_currentScript(self, newval: str) -> int:
        """
        Stops current running script, and/or selects a script to run immediately in a
        fresh new environment. If the MicroPython interpreter is busy running a script,
        this function will abort it immediately and reset the execution environment.
        If a non-empty string is given as argument, the new script will be started.

        @param newval : a string

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        rest_val = newval
        return await self._setAttr("currentScript", rest_val)

    async def get_startupScript(self) -> str:
        """
        Returns the name of the script to run when the device is powered on.

        @return a string corresponding to the name of the script to run when the device is powered on

        On failure, throws an exception or returns YMicroPython.STARTUPSCRIPT_INVALID.
        """
        res: str
        if self._cacheExpiration <= YAPI.GetTickCount():
            if await self.load(self._yapi.GetCacheValidity()) != YAPI.SUCCESS:
                return YMicroPython.STARTUPSCRIPT_INVALID
        res = self._startupScript
        return res

    async def set_startupScript(self, newval: str) -> int:
        """
        Changes the script to run when the device is powered on.
        Remember to call the saveToFlash() method of the module if the
        modification must be kept.

        @param newval : a string corresponding to the script to run when the device is powered on

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        rest_val = newval
        return await self._setAttr("startupScript", rest_val)

    async def get_debugMode(self) -> int:
        """
        Returns the activation state of micropython debugging interface.

        @return either YMicroPython.DEBUGMODE_OFF or YMicroPython.DEBUGMODE_ON, according to the activation
        state of micropython debugging interface

        On failure, throws an exception or returns YMicroPython.DEBUGMODE_INVALID.
        """
        res: int
        if self._cacheExpiration <= YAPI.GetTickCount():
            if await self.load(self._yapi.GetCacheValidity()) != YAPI.SUCCESS:
                return YMicroPython.DEBUGMODE_INVALID
        res = self._debugMode
        return res

    async def set_debugMode(self, newval: int) -> int:
        """
        Changes the activation state of micropython debugging interface.

        @param newval : either YMicroPython.DEBUGMODE_OFF or YMicroPython.DEBUGMODE_ON, according to the
        activation state of micropython debugging interface

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        rest_val = "1" if newval > 0 else "0"
        return await self._setAttr("debugMode", rest_val)

    async def get_command(self) -> str:
        res: str
        if self._cacheExpiration <= YAPI.GetTickCount():
            if await self.load(self._yapi.GetCacheValidity()) != YAPI.SUCCESS:
                return YMicroPython.COMMAND_INVALID
        res = self._command
        return res

    async def set_command(self, newval: str) -> int:
        rest_val = newval
        return await self._setAttr("command", rest_val)

    @staticmethod
    def FindMicroPython(func: str) -> YMicroPython:
        """
        Retrieves a MicroPython interpreter for a given identifier.
        The identifier can be specified using several formats:

        - FunctionLogicalName
        - ModuleSerialNumber.FunctionIdentifier
        - ModuleSerialNumber.FunctionLogicalName
        - ModuleLogicalName.FunctionIdentifier
        - ModuleLogicalName.FunctionLogicalName


        This function does not require that the MicroPython interpreter is online at the time
        it is invoked. The returned object is nevertheless valid.
        Use the method YMicroPython.isOnline() to test if the MicroPython interpreter is
        indeed online at a given time. In case of ambiguity when looking for
        a MicroPython interpreter by logical name, no error is notified: the first instance
        found is returned. The search is performed first by hardware name,
        then by logical name.

        If a call to this object's is_online() method returns FALSE although
        you are certain that the matching device is plugged, make sure that you did
        call registerHub() at application initialization time.

        @param func : a string that uniquely characterizes the MicroPython interpreter, for instance
                MyDevice.microPython.

        @return a YMicroPython object allowing you to drive the MicroPython interpreter.
        """
        obj: Union[YMicroPython, None]
        obj = YFunction._FindFromCache("MicroPython", func)
        if obj is None:
            obj = YMicroPython(YAPI, func)
            YFunction._AddToCache("MicroPython", func, obj)
        return obj

    @staticmethod
    def FindMicroPythonInContext(yctx: YAPIContext, func: str) -> YMicroPython:
        """
        Retrieves a MicroPython interpreter for a given identifier in a YAPI context.
        The identifier can be specified using several formats:

        - FunctionLogicalName
        - ModuleSerialNumber.FunctionIdentifier
        - ModuleSerialNumber.FunctionLogicalName
        - ModuleLogicalName.FunctionIdentifier
        - ModuleLogicalName.FunctionLogicalName


        This function does not require that the MicroPython interpreter is online at the time
        it is invoked. The returned object is nevertheless valid.
        Use the method YMicroPython.isOnline() to test if the MicroPython interpreter is
        indeed online at a given time. In case of ambiguity when looking for
        a MicroPython interpreter by logical name, no error is notified: the first instance
        found is returned. The search is performed first by hardware name,
        then by logical name.

        @param yctx : a YAPI context
        @param func : a string that uniquely characterizes the MicroPython interpreter, for instance
                MyDevice.microPython.

        @return a YMicroPython object allowing you to drive the MicroPython interpreter.
        """
        obj: Union[YMicroPython, None]
        obj = YFunction._FindFromCacheInContext(yctx, "MicroPython", func)
        if obj is None:
            obj = YMicroPython(yctx, func)
            YFunction._AddToCache("MicroPython", func, obj)
        return obj

    if not _IS_MICROPYTHON:
        async def registerValueCallback(self, callback: YMicroPythonValueCallback) -> int:
            """
            Registers the callback function that is invoked on every change of advertised value.
            The callback is invoked only during the execution of ySleep or yHandleEvents.
            This provides control over the time when the callback is triggered. For good responsiveness, remember to call
            one of these two functions periodically. To unregister a callback, pass a None pointer as argument.

            @param callback : the callback function to call, or a None pointer. The callback function should take two
                    arguments: the function object of which the value has changed, and the character string describing
                    the new advertised value.
            @noreturn
            """
            return await super().registerValueCallback(callback)

    async def eval(self, codeName: str, mpyCode: str) -> int:
        """
        Submit MicroPython code for execution in the interpreter.
        If the MicroPython interpreter is busy, this function will
        block until it becomes available. The code is then uploaded,
        compiled and executed on the fly, without beeing stored on the device filesystem.

        There is no implicit reset of the MicroPython interpreter with
        this function. Use method reset() if you need to start
        from a fresh environment to run your code.

        Note that although MicroPython is mostly compatible with recent Python 3.x
        interpreters, the limited ressources on the device impose some restrictions,
        in particular regarding the libraries that can be used. Please refer to
        the documentation for more details.

        @param codeName : name of the code file (used for error reporting only)
        @param mpyCode : MicroPython code to compile and execute

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        fullname: str
        res: int
        fullname = "mpy:%s" % codeName
        res = await self._upload(fullname, xbytearray(mpyCode, 'latin-1'))
        return res

    async def reset(self) -> int:
        """
        Stops current execution, and reset the MicroPython interpreter to initial state.
        All global variables are cleared, and all imports are forgotten.

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        res: int
        state: str

        res = await self.set_command("Z")
        if not (res == YAPI.SUCCESS):
            self._throw(YAPI.IO_ERROR, "unable to trigger MicroPython reset")
            return YAPI.IO_ERROR
        # Wait until the reset is effective
        state = (await self.get_advertisedValue())[0: 0 + 1]
        while not (state == "z"):
            await YAPI.Sleep(50)
            state = (await self.get_advertisedValue())[0: 0 + 1]
        return YAPI.SUCCESS

    async def get_lastLogs(self) -> str:
        """
        Returns a string with last logs of the MicroPython interpreter.
        This method return only logs that are still in the module.

        @return a string with last MicroPython logs.
                On failure, throws an exception or returns  YAPI.INVALID_STRING.
        """
        buff: xarray
        bufflen: int
        res: str

        buff = await self._download("mpy.txt")
        bufflen = len(buff) - 1
        while (bufflen > 0) and(buff[bufflen] != 64):
            bufflen = bufflen - 1
        res = (buff.decode('latin-1'))[0: 0 + bufflen]
        return res

    async def registerLogCallback(self, callback: YMicroPythonLogCallback) -> int:
        """
        Registers a device log callback function. This callback will be called each time
        microPython sends a new log message.

        @param callback : the callback function to invoke, or a None pointer.
                The callback function should take two arguments:
                the module object that emitted the log message,
                and the character string containing the log.
                On failure, throws an exception or returns a negative error code.
        """
        serial: str

        serial = await self.get_serialNumber()
        if serial == YAPI.INVALID_STRING:
            return YAPI.DEVICE_NOT_FOUND
        self._logCallback = callback
        self._isFirstCb = True
        if callback:
            await self.registerValueCallback(yInternalEventCallback)
        else:
            await self.registerValueCallback(None)
        return 0

    def get_logCallback(self) -> YMicroPythonLogCallback:
        return self._logCallback

    async def _internalEventHandler(self, cbVal: str) -> int:
        cbPos: int
        cbDPos: int
        url: str
        content: xarray
        endPos: int
        contentStr: str
        msgArr: list[str] = []
        arrLen: int
        lenStr: str
        arrPos: int
        logMsg: str
        # detect possible power cycle of the reader to clear event pointer
        cbPos = int(cbVal[1: 1 + len(cbVal)-1], 16)
        cbDPos = ((cbPos - self._prevCbPos) & 0xfffff)
        self._prevCbPos = cbPos
        if cbDPos > 65536:
            self._logPos = 0
        if not (self._logCallback):
            return YAPI.SUCCESS
        if self._isFirstCb:
            # use first emulated value callback caused by registerValueCallback:
            # to retrieve current logs position
            self._logPos = 0
            self._prevPartialLog = ""
            url = "mpy.txt"
        else:
            # load all messages since previous call
            url = "mpy.txt?pos=%d" % self._logPos

        content = await self._download(url)
        contentStr = content.decode('latin-1')
        # look for new position indicator at end of logs
        endPos = len(content) - 1
        while (endPos >= 0) and(content[endPos] != 64):
            endPos = endPos - 1
        if not (endPos > 0):
            self._throw(YAPI.IO_ERROR, "fail to download micropython logs")
            return YAPI.IO_ERROR
        lenStr = contentStr[endPos+1: endPos+1 + len(contentStr)-(endPos+1)]
        # update processed event position pointer
        self._logPos = YAPI._atoi(lenStr)
        if self._isFirstCb:
            # don't generate callbacks log messages before call to registerLogCallback
            self._isFirstCb = False
            return YAPI.SUCCESS
        # now generate callbacks for each complete log line
        endPos = endPos - 1
        if not (content[endPos] == 10):
            self._throw(YAPI.IO_ERROR, "fail to download micropython logs")
            return YAPI.IO_ERROR
        contentStr = contentStr[0: 0 + endPos]
        msgArr = (contentStr).split('\n')
        arrLen = len(msgArr) - 1
        if arrLen > 0:
            logMsg = "%s%s" % (self._prevPartialLog, msgArr[0])
            if self._logCallback:
                try:
                    retval = self._logCallback(self, logMsg)
                    if retval is not None: await retval
                # noinspection PyBroadException
                except Exception as e:
                    print('Exception in %s.logCallback:' % type(self).__name__, type(e).__name__, e)
            self._prevPartialLog = ""
            arrPos = 1
            while arrPos < arrLen:
                logMsg = msgArr[arrPos]
                if self._logCallback:
                    try:
                        retval = self._logCallback(self, logMsg)
                        if retval is not None: await retval
                    # noinspection PyBroadException
                    except Exception as e:
                        print('Exception in %s.logCallback:' % type(self).__name__, type(e).__name__, e)
                arrPos = arrPos + 1
        self._prevPartialLog = "%s%s" % (self._prevPartialLog, msgArr[arrLen])
        return YAPI.SUCCESS

    # --- (end of YMicroPython implementation)




#!/usr/bin/python
# -*- coding: utf-8 -*-
# add ../../Sources to the PYTHONPATH
import sys
import os
import time

sys.path.append(os.path.join("..", "..", "Sources"))
from yocto_api import *


def dumpSensor(sensor):
    fmt = "%d %b %Y %H:%M:%S,%f"
    print("Using DataLogger of " + sensor.get_friendlyName())
    dataset = sensor.get_recordedData(0, 0)
    print("loading summary... ")
    dataset.loadMore()
    summary = dataset.get_summary()
    print("from %s to %s : min=%.3f%s avg=%.3f%s  max=%.3f%s" % (
        summary.get_startTimeUTC_asDatetime().strftime(fmt),
        summary.get_endTimeUTC_asDatetime().strftime(fmt),
        summary.get_minValue(), sensor.get_unit(),
        summary.get_averageValue(), sensor.get_unit(),
        summary.get_maxValue(), sensor.get_unit()))
    print("loading details :   0%")
    progress = 0
    while progress < 100:
        progress = dataset.loadMore()
        # print("\b\b\b\b%3d%%" % progress)
    details = dataset.get_measures()
    for measure in details:
        print("from %s to %s : min=%.3f%s avg=%.3f%s  max=%.3f%s" % (
            measure.get_startTimeUTC_asDatetime().strftime(fmt),
            measure.get_endTimeUTC_asDatetime().strftime(fmt),
            measure.get_minValue(), sensor.get_unit(),
            measure.get_averageValue(), sensor.get_unit(),
            measure.get_maxValue(), sensor.get_unit()))


def main():
    errmsg = YRefParam()
    # Setup the API to use local USB devices
    if YAPI.RegisterHub("usb", errmsg) != YAPI.SUCCESS:
        sys.exit("init error" + errmsg.value)

    if len(sys.argv) == 1 or sys.argv[1] == 'any':
        sensor = YSensor.FirstSensor()
        if sensor is None:
            sys.exit("No module connected (check USB cable)")
    else:
        sensor = YSensor.FindSensor(sys.argv[0])
        if not sensor.isOnline():
            sys.exit("Sensor " + sensor.get_hardwareId() + " is not connected (check USB cable)")
    dumpSensor(sensor)
    YAPI.FreeAPI()


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
# ********************************************************************
#
#  $Id: svn_id $
#
#  Asyncio implementation of YLatitude
#
#  - - - - - - - - - License information: - - - - - - - - -
#
#  Copyright (C) 2011 and beyond by Yoctopuce Sarl, Switzerland.
#
#  Yoctopuce Sarl (hereafter Licensor) grants to you a perpetual
#  non-exclusive license to use, modify, copy and integrate this
#  file into your software for the sole purpose of interfacing
#  with Yoctopuce products.
#
#  You may reproduce and distribute copies of this file in
#  source or object form, as long as the sole purpose of this
#  code is to interface with Yoctopuce products. You must retain
#  this notice in the distributed source file.
#
#  You should refer to Yoctopuce General Terms and Conditions
#  for additional information regarding your rights and
#  obligations.
#
#  THE SOFTWARE AND DOCUMENTATION ARE PROVIDED 'AS IS' WITHOUT
#  WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING
#  WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, FITNESS
#  FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO
#  EVENT SHALL LICENSOR BE LIABLE FOR ANY INCIDENTAL, SPECIAL,
#  INDIRECT OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA,
#  COST OF PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR
#  SERVICES, ANY CLAIMS BY THIRD PARTIES (INCLUDING BUT NOT
#  LIMITED TO ANY DEFENSE THEREOF), ANY CLAIMS FOR INDEMNITY OR
#  CONTRIBUTION, OR OTHER SIMILAR COSTS, WHETHER ASSERTED ON THE
#  BASIS OF CONTRACT, TORT (INCLUDING NEGLIGENCE), BREACH OF
#  WARRANTY, OR OTHERWISE.
#
# *********************************************************************
"""
Yoctopuce library: Asyncio implementation of YLatitude
version: PATCH_WITH_VERSION
requires: yocto_api_aio
"""
from __future__ import annotations

import sys

# On MicroPython, code below will be wiped out at compile time
if sys.implementation.name != "micropython":
    # In CPython, enable edit-time type checking, including Final declaration
    from typing import Any, Union
    from collections.abc import Callable, Awaitable
    from .yocto_api_aio import const, _IS_MICROPYTHON
else:
    # In our micropython VM, common generic types are global built-ins
    # Others such as TypeVar should be avoided when using micropython,
    # as they produce overhead in runtime code
    # Final is translated into const() expressions before compilation
    _IS_MICROPYTHON: Final[bool] = True # noqa

from .yocto_api_aio import (
    YAPIContext, YAPI, YAPI_Exception, HwId, hwid2str, YFunction, YSensor, YMeasure
)

# --- (YLatitude class start)
if not _IS_MICROPYTHON:
    # For CPython, use strongly typed callback types
    try:
        YLatitudeValueCallback = Union[Callable[['YLatitude', str], Awaitable[None]], None]
        YLatitudeTimedReportCallback = Union[Callable[['YLatitude', YMeasure], Awaitable[None]], None]
    except TypeError:
        YLatitudeValueCallback = Union[Callable, Awaitable]
        YLatitudeTimedReportCallback = Union[Callable, Awaitable]

# noinspection PyProtectedMember
class YLatitude(YSensor):
    """
    The YLatitude class allows you to read and configure Yoctopuce latitude sensors.
    It inherits from YSensor class the core functions to read measures,
    to register callback functions, and to access the autonomous datalogger.

    """
    # --- (end of YLatitude class start)
    if not _IS_MICROPYTHON:
        # --- (YLatitude return codes)
        pass
        # --- (end of YLatitude return codes)

    # --- (YLatitude attributes declaration)
    _valueCallback: YLatitudeValueCallback
    _timedReportCallback: YLatitudeTimedReportCallback
    # --- (end of YLatitude attributes declaration)


    def __init__(self, yctx: YAPIContext, func: str):
        super().__init__(yctx, func)
        self._className = 'Latitude'
        # --- (YLatitude constructor)
        # --- (end of YLatitude constructor)

    # --- (YLatitude implementation)

    @staticmethod
    def FirstLatitude() -> Union[YLatitude, None]:
        """
        Starts the enumeration of latitude sensors currently accessible.
        Use the method YLatitude.nextLatitude() to iterate on
        next latitude sensors.

        @return a pointer to a YLatitude object, corresponding to
                the first latitude sensor currently online, or a None pointer
                if there are none.
        """
        next_hwid: Union[HwId, None] = YAPI._yHash.getFirstHardwareId('Latitude')
        if not next_hwid:
            return None
        return YLatitude.FindLatitude(hwid2str(next_hwid))

    @staticmethod
    def FirstLatitudeInContext(yctx: YAPIContext) -> Union[YLatitude, None]:
        """
        Starts the enumeration of latitude sensors currently accessible.
        Use the method YLatitude.nextLatitude() to iterate on
        next latitude sensors.

        @param yctx : a YAPI context.

        @return a pointer to a YLatitude object, corresponding to
                the first latitude sensor currently online, or a None pointer
                if there are none.
        """
        next_hwid: Union[HwId, None] = yctx._yHash.getFirstHardwareId('Latitude')
        if not next_hwid:
            return None
        return YLatitude.FindLatitudeInContext(yctx, hwid2str(next_hwid))

    def nextLatitude(self):
        """
        Continues the enumeration of latitude sensors started using yFirstLatitude().
        Caution: You can't make any assumption about the returned latitude sensors order.
        If you want to find a specific a latitude sensor, use Latitude.findLatitude()
        and a hardwareID or a logical name.

        @return a pointer to a YLatitude object, corresponding to
                a latitude sensor currently online, or a None pointer
                if there are no more latitude sensors to enumerate.
        """
        next_hwid: Union[HwId, None] = None
        try:
            hwid: HwId = self._yapi._yHash.resolveHwID(self._className, self._func)
            next_hwid = self._yapi._yHash.getNextHardwareId(self._className, hwid)
        except YAPI_Exception:
            pass
        if not next_hwid:
            return None
        return YLatitude.FindLatitudeInContext(self._yapi, hwid2str(next_hwid))

    def _parseAttr(self, json_val: dict) -> None:
        super()._parseAttr(json_val)

    @staticmethod
    def FindLatitude(func: str) -> YLatitude:
        """
        Retrieves a latitude sensor for a given identifier.
        The identifier can be specified using several formats:

        - FunctionLogicalName
        - ModuleSerialNumber.FunctionIdentifier
        - ModuleSerialNumber.FunctionLogicalName
        - ModuleLogicalName.FunctionIdentifier
        - ModuleLogicalName.FunctionLogicalName


        This function does not require that the latitude sensor is online at the time
        it is invoked. The returned object is nevertheless valid.
        Use the method YLatitude.isOnline() to test if the latitude sensor is
        indeed online at a given time. In case of ambiguity when looking for
        a latitude sensor by logical name, no error is notified: the first instance
        found is returned. The search is performed first by hardware name,
        then by logical name.

        If a call to this object's is_online() method returns FALSE although
        you are certain that the matching device is plugged, make sure that you did
        call registerHub() at application initialization time.

        @param func : a string that uniquely characterizes the latitude sensor, for instance
                YGNSSMK2.latitude.

        @return a YLatitude object allowing you to drive the latitude sensor.
        """
        obj: Union[YLatitude, None]
        obj = YFunction._FindFromCache("Latitude", func)
        if obj is None:
            obj = YLatitude(YAPI, func)
            YFunction._AddToCache("Latitude", func, obj)
        return obj

    @staticmethod
    def FindLatitudeInContext(yctx: YAPIContext, func: str) -> YLatitude:
        """
        Retrieves a latitude sensor for a given identifier in a YAPI context.
        The identifier can be specified using several formats:

        - FunctionLogicalName
        - ModuleSerialNumber.FunctionIdentifier
        - ModuleSerialNumber.FunctionLogicalName
        - ModuleLogicalName.FunctionIdentifier
        - ModuleLogicalName.FunctionLogicalName


        This function does not require that the latitude sensor is online at the time
        it is invoked. The returned object is nevertheless valid.
        Use the method YLatitude.isOnline() to test if the latitude sensor is
        indeed online at a given time. In case of ambiguity when looking for
        a latitude sensor by logical name, no error is notified: the first instance
        found is returned. The search is performed first by hardware name,
        then by logical name.

        @param yctx : a YAPI context
        @param func : a string that uniquely characterizes the latitude sensor, for instance
                YGNSSMK2.latitude.

        @return a YLatitude object allowing you to drive the latitude sensor.
        """
        obj: Union[YLatitude, None]
        obj = YFunction._FindFromCacheInContext(yctx, "Latitude", func)
        if obj is None:
            obj = YLatitude(yctx, func)
            YFunction._AddToCache("Latitude", func, obj)
        return obj

    if not _IS_MICROPYTHON:
        async def registerValueCallback(self, callback: YLatitudeValueCallback) -> int:
            """
            Registers the callback function that is invoked on every change of advertised value.
            The callback is invoked only during the execution of ySleep or yHandleEvents.
            This provides control over the time when the callback is triggered. For good responsiveness, remember to call
            one of these two functions periodically. To unregister a callback, pass a None pointer as argument.

            @param callback : the callback function to call, or a None pointer. The callback function should take two
                    arguments: the function object of which the value has changed, and the character string describing
                    the new advertised value.
            @noreturn
            """
            return await super().registerValueCallback(callback)

    if not _IS_MICROPYTHON:
        async def registerTimedReportCallback(self, callback: YLatitudeTimedReportCallback) -> int:
            """
            Registers the callback function that is invoked on every periodic timed notification.
            The callback is invoked only during the execution of ySleep or yHandleEvents.
            This provides control over the time when the callback is triggered. For good responsiveness, remember to call
            one of these two functions periodically. To unregister a callback, pass a None pointer as argument.

            @param callback : the callback function to call, or a None pointer. The callback function should take two
                    arguments: the function object of which the value has changed, and an YMeasure object describing
                    the new advertised value.
            @noreturn
            """
            return await super().registerTimedReportCallback(callback)

    # --- (end of YLatitude implementation)




# -*- coding: utf-8 -*-
# ********************************************************************
#
#  $Id: svn_id $
#
#  High-level API for YMotor
#
#  - - - - - - - - - License information: - - - - - - - - -
#
#  Copyright (C) 2011 and beyond by Yoctopuce Sarl, Switzerland.
#
#  Yoctopuce Sarl (hereafter Licensor) grants to you a perpetual
#  non-exclusive license to use, modify, copy and integrate this
#  file into your software for the sole purpose of interfacing
#  with Yoctopuce products.
#
#  You may reproduce and distribute copies of this file in
#  source or object form, as long as the sole purpose of this
#  code is to interface with Yoctopuce products. You must retain
#  this notice in the distributed source file.
#
#  You should refer to Yoctopuce General Terms and Conditions
#  for additional information regarding your rights and
#  obligations.
#
#  THE SOFTWARE AND DOCUMENTATION ARE PROVIDED 'AS IS' WITHOUT
#  WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING
#  WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, FITNESS
#  FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO
#  EVENT SHALL LICENSOR BE LIABLE FOR ANY INCIDENTAL, SPECIAL,
#  INDIRECT OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA,
#  COST OF PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR
#  SERVICES, ANY CLAIMS BY THIRD PARTIES (INCLUDING BUT NOT
#  LIMITED TO ANY DEFENSE THEREOF), ANY CLAIMS FOR INDEMNITY OR
#  CONTRIBUTION, OR OTHER SIMILAR COSTS, WHETHER ASSERTED ON THE
#  BASIS OF CONTRACT, TORT (INCLUDING NEGLIGENCE), BREACH OF
#  WARRANTY, OR OTHERWISE.
#
# *********************************************************************
"""
Yoctopuce library: High-level API for YMotor
version: PATCH_WITH_VERSION
requires: yocto_motor_aio
requires: yocto_api
"""
from __future__ import annotations

import sys

# On MicroPython, code below will be wiped out at compile time
if sys.implementation.name != "micropython":
    # In CPython, enable edit-time type checking, including Final declaration
    from typing import Union, Final
    from collections.abc import Callable, Awaitable
    from .yocto_api import _IS_MICROPYTHON, _DYNAMIC_HELPERS
else:
    # In our micropython VM, common generic types are global built-ins
    # Others such as TypeVar should be avoided when using micropython,
    # as they produce overhead in runtime code
    # Final is translated into const() expressions before compilation
    _IS_MICROPYTHON: Final[bool] = True # noqa
    _DYNAMIC_HELPERS: Final[bool] = True # noqa

from .yocto_motor_aio import YMotor as YMotor_aio
from .yocto_api import (
    YAPIContext, YAPI, YFunction
)

# --- (YMotor class start)
if not _IS_MICROPYTHON:
    # For CPython, use strongly typed callback types
    try:
        YMotorValueCallback = Union[Callable[['YMotor', str], Awaitable[None]], None]
    except TypeError:
        YMotorValueCallback = Union[Callable, Awaitable]

# noinspection PyProtectedMember
class YMotor(YFunction):
    """
    The YMotor class allows you to drive a DC motor. It can be used to configure the
    power sent to the motor to make it turn both ways, but also to drive accelerations
    and decelerations. The motor will then accelerate automatically: you will not
    have to monitor it. The API also allows to slow down the motor by shortening
    its terminals: the motor will then act as an electromagnetic brake.

    """
    _aio: YMotor_aio
    # --- (end of YMotor class start)
    if not _IS_MICROPYTHON:
        # --- (YMotor return codes)
        DRIVINGFORCE_INVALID: Final[float] = YAPI.INVALID_DOUBLE
        BRAKINGFORCE_INVALID: Final[float] = YAPI.INVALID_DOUBLE
        CUTOFFVOLTAGE_INVALID: Final[float] = YAPI.INVALID_DOUBLE
        OVERCURRENTLIMIT_INVALID: Final[int] = YAPI.INVALID_UINT
        FREQUENCY_INVALID: Final[float] = YAPI.INVALID_DOUBLE
        STARTERTIME_INVALID: Final[int] = YAPI.INVALID_UINT
        FAILSAFETIMEOUT_INVALID: Final[int] = YAPI.INVALID_UINT
        COMMAND_INVALID: Final[str] = YAPI.INVALID_STRING
        MOTORSTATUS_IDLE: Final[int] = 0
        MOTORSTATUS_BRAKE: Final[int] = 1
        MOTORSTATUS_FORWD: Final[int] = 2
        MOTORSTATUS_BACKWD: Final[int] = 3
        MOTORSTATUS_LOVOLT: Final[int] = 4
        MOTORSTATUS_HICURR: Final[int] = 5
        MOTORSTATUS_HIHEAT: Final[int] = 6
        MOTORSTATUS_FAILSF: Final[int] = 7
        MOTORSTATUS_INVALID: Final[int] = -1
        # --- (end of YMotor return codes)


    # --- (YMotor implementation)

    @classmethod
    def FirstMotor(cls) -> Union[YMotor, None]:
        """
        Starts the enumeration of motors currently accessible.
        Use the method YMotor.nextMotor() to iterate on
        next motors.

        @return a pointer to a YMotor object, corresponding to
                the first motor currently online, or a None pointer
                if there are none.
        """
        return cls._proxy(cls, YMotor_aio.FirstMotor())

    @classmethod
    def FirstMotorInContext(cls, yctx: YAPIContext) -> Union[YMotor, None]:
        """
        Starts the enumeration of motors currently accessible.
        Use the method YMotor.nextMotor() to iterate on
        next motors.

        @param yctx : a YAPI context.

        @return a pointer to a YMotor object, corresponding to
                the first motor currently online, or a None pointer
                if there are none.
        """
        return cls._proxy(cls, YMotor_aio.FirstMotorInContext(yctx))

    def nextMotor(self):
        """
        Continues the enumeration of motors started using yFirstMotor().
        Caution: You can't make any assumption about the returned motors order.
        If you want to find a specific a motor, use Motor.findMotor()
        and a hardwareID or a logical name.

        @return a pointer to a YMotor object, corresponding to
                a motor currently online, or a None pointer
                if there are no more motors to enumerate.
        """
        return self._proxy(type(self), self._aio.nextMotor())

    if not _DYNAMIC_HELPERS:
        def get_motorStatus(self) -> int:
            """
            Return the controller state. Possible states are:
            IDLE   when the motor is stopped/in free wheel, ready to start;
            FORWD  when the controller is driving the motor forward;
            BACKWD when the controller is driving the motor backward;
            BRAKE  when the controller is braking;
            LOVOLT when the controller has detected a low voltage condition;
            HICURR when the controller has detected an over current condition;
            HIHEAT when the controller has detected an overheat condition;
            FAILSF when the controller switched on the failsafe security.

            When an error condition occurred (LOVOLT, HICURR, HIHEAT, FAILSF), the controller
            status must be explicitly reset using the resetStatus function.

            @return a value among YMotor.MOTORSTATUS_IDLE, YMotor.MOTORSTATUS_BRAKE, YMotor.MOTORSTATUS_FORWD,
            YMotor.MOTORSTATUS_BACKWD, YMotor.MOTORSTATUS_LOVOLT, YMotor.MOTORSTATUS_HICURR,
            YMotor.MOTORSTATUS_HIHEAT and YMotor.MOTORSTATUS_FAILSF

            On failure, throws an exception or returns YMotor.MOTORSTATUS_INVALID.
            """
            return self._run(self._aio.get_motorStatus())

    if not _DYNAMIC_HELPERS:
        def set_motorStatus(self, newval: int) -> int:
            return self._run(self._aio.set_motorStatus(newval))

    if not _DYNAMIC_HELPERS:
        def set_drivingForce(self, newval: float) -> int:
            """
            Changes immediately the power sent to the motor. The value is a percentage between -100%
            to 100%. If you want go easy on your mechanics and avoid excessive current consumption,
            try to avoid brutal power changes. For example, immediate transition from forward full power
            to reverse full power is a very bad idea. Each time the driving power is modified, the
            braking power is set to zero.

            @param newval : a floating point number corresponding to immediately the power sent to the motor

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_drivingForce(newval))

    if not _DYNAMIC_HELPERS:
        def get_drivingForce(self) -> float:
            """
            Returns the power sent to the motor, as a percentage between -100% and +100%.

            @return a floating point number corresponding to the power sent to the motor, as a percentage
            between -100% and +100%

            On failure, throws an exception or returns YMotor.DRIVINGFORCE_INVALID.
            """
            return self._run(self._aio.get_drivingForce())

    if not _DYNAMIC_HELPERS:
        def set_brakingForce(self, newval: float) -> int:
            """
            Changes immediately the braking force applied to the motor (in percents).
            The value 0 corresponds to no braking (free wheel). When the braking force
            is changed, the driving power is set to zero. The value is a percentage.

            @param newval : a floating point number corresponding to immediately the braking force applied to
            the motor (in percents)

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_brakingForce(newval))

    if not _DYNAMIC_HELPERS:
        def get_brakingForce(self) -> float:
            """
            Returns the braking force applied to the motor, as a percentage.
            The value 0 corresponds to no braking (free wheel).

            @return a floating point number corresponding to the braking force applied to the motor, as a percentage

            On failure, throws an exception or returns YMotor.BRAKINGFORCE_INVALID.
            """
            return self._run(self._aio.get_brakingForce())

    if not _DYNAMIC_HELPERS:
        def set_cutOffVoltage(self, newval: float) -> int:
            """
            Changes the threshold voltage under which the controller automatically switches to error state
            and prevents further current draw. This setting prevent damage to a battery that can
            occur when drawing current from an "empty" battery.
            Note that whatever the cutoff threshold, the controller switches to undervoltage
            error state if the power supply goes under 3V, even for a very brief time.
            Remember to call the saveToFlash()
            method of the module if the modification must be kept.

            @param newval : a floating point number corresponding to the threshold voltage under which the
            controller automatically switches to error state
                    and prevents further current draw

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_cutOffVoltage(newval))

    if not _DYNAMIC_HELPERS:
        def get_cutOffVoltage(self) -> float:
            """
            Returns the threshold voltage under which the controller automatically switches to error state
            and prevents further current draw. This setting prevents damage to a battery that can
            occur when drawing current from an "empty" battery.

            @return a floating point number corresponding to the threshold voltage under which the controller
            automatically switches to error state
                    and prevents further current draw

            On failure, throws an exception or returns YMotor.CUTOFFVOLTAGE_INVALID.
            """
            return self._run(self._aio.get_cutOffVoltage())

    if not _DYNAMIC_HELPERS:
        def get_overCurrentLimit(self) -> int:
            """
            Returns the current threshold (in mA) above which the controller automatically
            switches to error state. A zero value means that there is no limit.

            @return an integer corresponding to the current threshold (in mA) above which the controller automatically
                    switches to error state

            On failure, throws an exception or returns YMotor.OVERCURRENTLIMIT_INVALID.
            """
            return self._run(self._aio.get_overCurrentLimit())

    if not _DYNAMIC_HELPERS:
        def set_overCurrentLimit(self, newval: int) -> int:
            """
            Changes the current threshold (in mA) above which the controller automatically
            switches to error state. A zero value means that there is no limit. Note that whatever the
            current limit is, the controller switches to OVERCURRENT status if the current
            goes above 32A, even for a very brief time. Remember to call the saveToFlash()
            method of the module if the modification must be kept.

            @param newval : an integer corresponding to the current threshold (in mA) above which the
            controller automatically
                    switches to error state

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_overCurrentLimit(newval))

    if not _DYNAMIC_HELPERS:
        def set_frequency(self, newval: float) -> int:
            """
            Changes the PWM frequency used to control the motor. Low frequency is usually
            more efficient and may help the motor to start, but an audible noise might be
            generated. A higher frequency reduces the noise, but more energy is converted
            into heat. Remember to call the saveToFlash()
            method of the module if the modification must be kept.

            @param newval : a floating point number corresponding to the PWM frequency used to control the motor

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_frequency(newval))

    if not _DYNAMIC_HELPERS:
        def get_frequency(self) -> float:
            """
            Returns the PWM frequency used to control the motor.

            @return a floating point number corresponding to the PWM frequency used to control the motor

            On failure, throws an exception or returns YMotor.FREQUENCY_INVALID.
            """
            return self._run(self._aio.get_frequency())

    if not _DYNAMIC_HELPERS:
        def get_starterTime(self) -> int:
            """
            Returns the duration (in ms) during which the motor is driven at low frequency to help
            it start up.

            @return an integer corresponding to the duration (in ms) during which the motor is driven at low
            frequency to help
                    it start up

            On failure, throws an exception or returns YMotor.STARTERTIME_INVALID.
            """
            return self._run(self._aio.get_starterTime())

    if not _DYNAMIC_HELPERS:
        def set_starterTime(self, newval: int) -> int:
            """
            Changes the duration (in ms) during which the motor is driven at low frequency to help
            it start up. Remember to call the saveToFlash()
            method of the module if the modification must be kept.

            @param newval : an integer corresponding to the duration (in ms) during which the motor is driven
            at low frequency to help
                    it start up

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_starterTime(newval))

    if not _DYNAMIC_HELPERS:
        def get_failSafeTimeout(self) -> int:
            """
            Returns the delay in milliseconds allowed for the controller to run autonomously without
            receiving any instruction from the control process. When this delay has elapsed,
            the controller automatically stops the motor and switches to FAILSAFE error.
            Failsafe security is disabled when the value is zero.

            @return an integer corresponding to the delay in milliseconds allowed for the controller to run
            autonomously without
                    receiving any instruction from the control process

            On failure, throws an exception or returns YMotor.FAILSAFETIMEOUT_INVALID.
            """
            return self._run(self._aio.get_failSafeTimeout())

    if not _DYNAMIC_HELPERS:
        def set_failSafeTimeout(self, newval: int) -> int:
            """
            Changes the delay in milliseconds allowed for the controller to run autonomously without
            receiving any instruction from the control process. When this delay has elapsed,
            the controller automatically stops the motor and switches to FAILSAFE error.
            Failsafe security is disabled when the value is zero.
            Remember to call the saveToFlash()
            method of the module if the modification must be kept.

            @param newval : an integer corresponding to the delay in milliseconds allowed for the controller to
            run autonomously without
                    receiving any instruction from the control process

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_failSafeTimeout(newval))

    if not _DYNAMIC_HELPERS:
        def set_command(self, newval: str) -> int:
            return self._run(self._aio.set_command(newval))

    @classmethod
    def FindMotor(cls, func: str) -> YMotor:
        """
        Retrieves a motor for a given identifier.
        The identifier can be specified using several formats:

        - FunctionLogicalName
        - ModuleSerialNumber.FunctionIdentifier
        - ModuleSerialNumber.FunctionLogicalName
        - ModuleLogicalName.FunctionIdentifier
        - ModuleLogicalName.FunctionLogicalName


        This function does not require that the motor is online at the time
        it is invoked. The returned object is nevertheless valid.
        Use the method YMotor.isOnline() to test if the motor is
        indeed online at a given time. In case of ambiguity when looking for
        a motor by logical name, no error is notified: the first instance
        found is returned. The search is performed first by hardware name,
        then by logical name.

        If a call to this object's is_online() method returns FALSE although
        you are certain that the matching device is plugged, make sure that you did
        call registerHub() at application initialization time.

        @param func : a string that uniquely characterizes the motor, for instance
                MOTORCTL.motor.

        @return a YMotor object allowing you to drive the motor.
        """
        return cls._proxy(cls, YMotor_aio.FindMotor(func))

    @classmethod
    def FindMotorInContext(cls, yctx: YAPIContext, func: str) -> YMotor:
        """
        Retrieves a motor for a given identifier in a YAPI context.
        The identifier can be specified using several formats:

        - FunctionLogicalName
        - ModuleSerialNumber.FunctionIdentifier
        - ModuleSerialNumber.FunctionLogicalName
        - ModuleLogicalName.FunctionIdentifier
        - ModuleLogicalName.FunctionLogicalName


        This function does not require that the motor is online at the time
        it is invoked. The returned object is nevertheless valid.
        Use the method YMotor.isOnline() to test if the motor is
        indeed online at a given time. In case of ambiguity when looking for
        a motor by logical name, no error is notified: the first instance
        found is returned. The search is performed first by hardware name,
        then by logical name.

        @param yctx : a YAPI context
        @param func : a string that uniquely characterizes the motor, for instance
                MOTORCTL.motor.

        @return a YMotor object allowing you to drive the motor.
        """
        return cls._proxy(cls, YMotor_aio.FindMotorInContext(yctx, func))

    if not _IS_MICROPYTHON:
        def registerValueCallback(self, callback: YMotorValueCallback) -> int:
            """
            Registers the callback function that is invoked on every change of advertised value.
            The callback is invoked only during the execution of ySleep or yHandleEvents.
            This provides control over the time when the callback is triggered. For good responsiveness, remember to call
            one of these two functions periodically. To unregister a callback, pass a None pointer as argument.

            @param callback : the callback function to call, or a None pointer. The callback function should take two
                    arguments: the function object of which the value has changed, and the character string describing
                    the new advertised value.
            @noreturn
            """
            return super().registerValueCallback(callback)

    if not _DYNAMIC_HELPERS:
        def keepALive(self) -> int:
            """
            Rearms the controller failsafe timer. When the motor is running and the failsafe feature
            is active, this function should be called periodically to prove that the control process
            is running properly. Otherwise, the motor is automatically stopped after the specified
            timeout. Calling a motor <i>set</i> function implicitly rearms the failsafe timer.
            """
            return self._run(self._aio.keepALive())

    if not _DYNAMIC_HELPERS:
        def resetStatus(self) -> int:
            """
            Reset the controller state to IDLE. This function must be invoked explicitly
            after any error condition is signaled.
            """
            return self._run(self._aio.resetStatus())

    if not _DYNAMIC_HELPERS:
        def drivingForceMove(self, targetPower: float, delay: int) -> int:
            """
            Changes progressively the power sent to the motor for a specific duration.

            @param targetPower : desired motor power, in percents (between -100% and +100%)
            @param delay : duration (in ms) of the transition

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.drivingForceMove(targetPower, delay))

    if not _DYNAMIC_HELPERS:
        def brakingForceMove(self, targetPower: float, delay: int) -> int:
            """
            Changes progressively the braking force applied to the motor for a specific duration.

            @param targetPower : desired braking force, in percents
            @param delay : duration (in ms) of the transition

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.brakingForceMove(targetPower, delay))

    # --- (end of YMotor implementation)




# -*- coding: utf-8 -*-
# ********************************************************************
#
#  $Id: svn_id $
#
#  High-level API for YWeighScale
#
#  - - - - - - - - - License information: - - - - - - - - -
#
#  Copyright (C) 2011 and beyond by Yoctopuce Sarl, Switzerland.
#
#  Yoctopuce Sarl (hereafter Licensor) grants to you a perpetual
#  non-exclusive license to use, modify, copy and integrate this
#  file into your software for the sole purpose of interfacing
#  with Yoctopuce products.
#
#  You may reproduce and distribute copies of this file in
#  source or object form, as long as the sole purpose of this
#  code is to interface with Yoctopuce products. You must retain
#  this notice in the distributed source file.
#
#  You should refer to Yoctopuce General Terms and Conditions
#  for additional information regarding your rights and
#  obligations.
#
#  THE SOFTWARE AND DOCUMENTATION ARE PROVIDED 'AS IS' WITHOUT
#  WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING
#  WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, FITNESS
#  FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO
#  EVENT SHALL LICENSOR BE LIABLE FOR ANY INCIDENTAL, SPECIAL,
#  INDIRECT OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA,
#  COST OF PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR
#  SERVICES, ANY CLAIMS BY THIRD PARTIES (INCLUDING BUT NOT
#  LIMITED TO ANY DEFENSE THEREOF), ANY CLAIMS FOR INDEMNITY OR
#  CONTRIBUTION, OR OTHER SIMILAR COSTS, WHETHER ASSERTED ON THE
#  BASIS OF CONTRACT, TORT (INCLUDING NEGLIGENCE), BREACH OF
#  WARRANTY, OR OTHERWISE.
#
# *********************************************************************
"""
Yoctopuce library: High-level API for YWeighScale
version: PATCH_WITH_VERSION
requires: yocto_weighscale_aio
requires: yocto_api
"""
from __future__ import annotations

import sys

# On MicroPython, code below will be wiped out at compile time
if sys.implementation.name != "micropython":
    # In CPython, enable edit-time type checking, including Final declaration
    from typing import Union, Final
    from collections.abc import Callable, Awaitable
    from .yocto_api import _IS_MICROPYTHON, _DYNAMIC_HELPERS
else:
    # In our micropython VM, common generic types are global built-ins
    # Others such as TypeVar should be avoided when using micropython,
    # as they produce overhead in runtime code
    # Final is translated into const() expressions before compilation
    _IS_MICROPYTHON: Final[bool] = True # noqa
    _DYNAMIC_HELPERS: Final[bool] = True # noqa

from .yocto_weighscale_aio import YWeighScale as YWeighScale_aio
from .yocto_api import (
    YAPIContext, YAPI, YSensor, YMeasure
)

# --- (YWeighScale class start)
if not _IS_MICROPYTHON:
    # For CPython, use strongly typed callback types
    try:
        YWeighScaleValueCallback = Union[Callable[['YWeighScale', str], Awaitable[None]], None]
        YWeighScaleTimedReportCallback = Union[Callable[['YWeighScale', YMeasure], Awaitable[None]], None]
    except TypeError:
        YWeighScaleValueCallback = Union[Callable, Awaitable]
        YWeighScaleTimedReportCallback = Union[Callable, Awaitable]

# noinspection PyProtectedMember
class YWeighScale(YSensor):
    """
    The YWeighScale class provides a weight measurement from a ratiometric sensor.
    It can be used to control the bridge excitation parameters, in order to avoid
    measure shifts caused by temperature variation in the electronics, and can also
    automatically apply an additional correction factor based on temperature to
    compensate for offsets in the load cell itself.

    """
    _aio: YWeighScale_aio
    # --- (end of YWeighScale class start)
    if not _IS_MICROPYTHON:
        # --- (YWeighScale return codes)
        TEMPAVGADAPTRATIO_INVALID: Final[float] = YAPI.INVALID_DOUBLE
        TEMPCHGADAPTRATIO_INVALID: Final[float] = YAPI.INVALID_DOUBLE
        COMPTEMPAVG_INVALID: Final[float] = YAPI.INVALID_DOUBLE
        COMPTEMPCHG_INVALID: Final[float] = YAPI.INVALID_DOUBLE
        COMPENSATION_INVALID: Final[float] = YAPI.INVALID_DOUBLE
        ZEROTRACKING_INVALID: Final[float] = YAPI.INVALID_DOUBLE
        COMMAND_INVALID: Final[str] = YAPI.INVALID_STRING
        EXCITATION_OFF: Final[int] = 0
        EXCITATION_DC: Final[int] = 1
        EXCITATION_AC: Final[int] = 2
        EXCITATION_INVALID: Final[int] = -1
        # --- (end of YWeighScale return codes)


    # --- (YWeighScale implementation)

    @classmethod
    def FirstWeighScale(cls) -> Union[YWeighScale, None]:
        """
        Starts the enumeration of weighing scale sensors currently accessible.
        Use the method YWeighScale.nextWeighScale() to iterate on
        next weighing scale sensors.

        @return a pointer to a YWeighScale object, corresponding to
                the first weighing scale sensor currently online, or a None pointer
                if there are none.
        """
        return cls._proxy(cls, YWeighScale_aio.FirstWeighScale())

    @classmethod
    def FirstWeighScaleInContext(cls, yctx: YAPIContext) -> Union[YWeighScale, None]:
        """
        Starts the enumeration of weighing scale sensors currently accessible.
        Use the method YWeighScale.nextWeighScale() to iterate on
        next weighing scale sensors.

        @param yctx : a YAPI context.

        @return a pointer to a YWeighScale object, corresponding to
                the first weighing scale sensor currently online, or a None pointer
                if there are none.
        """
        return cls._proxy(cls, YWeighScale_aio.FirstWeighScaleInContext(yctx))

    def nextWeighScale(self):
        """
        Continues the enumeration of weighing scale sensors started using yFirstWeighScale().
        Caution: You can't make any assumption about the returned weighing scale sensors order.
        If you want to find a specific a weighing scale sensor, use WeighScale.findWeighScale()
        and a hardwareID or a logical name.

        @return a pointer to a YWeighScale object, corresponding to
                a weighing scale sensor currently online, or a None pointer
                if there are no more weighing scale sensors to enumerate.
        """
        return self._proxy(type(self), self._aio.nextWeighScale())

    if not _DYNAMIC_HELPERS:
        def set_unit(self, newval: str) -> int:
            """
            Changes the measuring unit for the weight.
            Remember to call the saveToFlash() method of the module if the
            modification must be kept.

            @param newval : a string corresponding to the measuring unit for the weight

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_unit(newval))

    if not _DYNAMIC_HELPERS:
        def get_excitation(self) -> int:
            """
            Returns the current load cell bridge excitation method.

            @return a value among YWeighScale.EXCITATION_OFF, YWeighScale.EXCITATION_DC and
            YWeighScale.EXCITATION_AC corresponding to the current load cell bridge excitation method

            On failure, throws an exception or returns YWeighScale.EXCITATION_INVALID.
            """
            return self._run(self._aio.get_excitation())

    if not _DYNAMIC_HELPERS:
        def set_excitation(self, newval: int) -> int:
            """
            Changes the current load cell bridge excitation method.
            Remember to call the saveToFlash() method of the module if the
            modification must be kept.

            @param newval : a value among YWeighScale.EXCITATION_OFF, YWeighScale.EXCITATION_DC and
            YWeighScale.EXCITATION_AC corresponding to the current load cell bridge excitation method

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_excitation(newval))

    if not _DYNAMIC_HELPERS:
        def set_tempAvgAdaptRatio(self, newval: float) -> int:
            """
            Changes the averaged temperature update rate, in per mille.
            The purpose of this adaptation ratio is to model the thermal inertia of the load cell.
            The averaged temperature is updated every 10 seconds, by applying this adaptation rate
            to the difference between the measures ambient temperature and the current compensation
            temperature. The standard rate is 0.2 per mille, and the maximal rate is 65 per mille.
            Remember to call the saveToFlash() method of the module if the
            modification must be kept.

            @param newval : a floating point number corresponding to the averaged temperature update rate, in per mille

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_tempAvgAdaptRatio(newval))

    if not _DYNAMIC_HELPERS:
        def get_tempAvgAdaptRatio(self) -> float:
            """
            Returns the averaged temperature update rate, in per mille.
            The purpose of this adaptation ratio is to model the thermal inertia of the load cell.
            The averaged temperature is updated every 10 seconds, by applying this adaptation rate
            to the difference between the measures ambient temperature and the current compensation
            temperature. The standard rate is 0.2 per mille, and the maximal rate is 65 per mille.

            @return a floating point number corresponding to the averaged temperature update rate, in per mille

            On failure, throws an exception or returns YWeighScale.TEMPAVGADAPTRATIO_INVALID.
            """
            return self._run(self._aio.get_tempAvgAdaptRatio())

    if not _DYNAMIC_HELPERS:
        def set_tempChgAdaptRatio(self, newval: float) -> int:
            """
            Changes the temperature change update rate, in per mille.
            The temperature change is updated every 10 seconds, by applying this adaptation rate
            to the difference between the measures ambient temperature and the current temperature used for
            change compensation. The standard rate is 0.6 per mille, and the maximal rate is 65 per mille.
            Remember to call the saveToFlash() method of the module if the
            modification must be kept.

            @param newval : a floating point number corresponding to the temperature change update rate, in per mille

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_tempChgAdaptRatio(newval))

    if not _DYNAMIC_HELPERS:
        def get_tempChgAdaptRatio(self) -> float:
            """
            Returns the temperature change update rate, in per mille.
            The temperature change is updated every 10 seconds, by applying this adaptation rate
            to the difference between the measures ambient temperature and the current temperature used for
            change compensation. The standard rate is 0.6 per mille, and the maximal rate is 65 per mille.

            @return a floating point number corresponding to the temperature change update rate, in per mille

            On failure, throws an exception or returns YWeighScale.TEMPCHGADAPTRATIO_INVALID.
            """
            return self._run(self._aio.get_tempChgAdaptRatio())

    if not _DYNAMIC_HELPERS:
        def get_compTempAvg(self) -> float:
            """
            Returns the current averaged temperature, used for thermal compensation.

            @return a floating point number corresponding to the current averaged temperature, used for thermal compensation

            On failure, throws an exception or returns YWeighScale.COMPTEMPAVG_INVALID.
            """
            return self._run(self._aio.get_compTempAvg())

    if not _DYNAMIC_HELPERS:
        def get_compTempChg(self) -> float:
            """
            Returns the current temperature variation, used for thermal compensation.

            @return a floating point number corresponding to the current temperature variation, used for
            thermal compensation

            On failure, throws an exception or returns YWeighScale.COMPTEMPCHG_INVALID.
            """
            return self._run(self._aio.get_compTempChg())

    if not _DYNAMIC_HELPERS:
        def get_compensation(self) -> float:
            """
            Returns the current current thermal compensation value.

            @return a floating point number corresponding to the current current thermal compensation value

            On failure, throws an exception or returns YWeighScale.COMPENSATION_INVALID.
            """
            return self._run(self._aio.get_compensation())

    if not _DYNAMIC_HELPERS:
        def set_zeroTracking(self, newval: float) -> int:
            """
            Changes the zero tracking threshold value. When this threshold is larger than
            zero, any measure under the threshold will automatically be ignored and the
            zero compensation will be updated.
            Remember to call the saveToFlash() method of the module if the
            modification must be kept.

            @param newval : a floating point number corresponding to the zero tracking threshold value

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_zeroTracking(newval))

    if not _DYNAMIC_HELPERS:
        def get_zeroTracking(self) -> float:
            """
            Returns the zero tracking threshold value. When this threshold is larger than
            zero, any measure under the threshold will automatically be ignored and the
            zero compensation will be updated.

            @return a floating point number corresponding to the zero tracking threshold value

            On failure, throws an exception or returns YWeighScale.ZEROTRACKING_INVALID.
            """
            return self._run(self._aio.get_zeroTracking())

    if not _DYNAMIC_HELPERS:
        def set_command(self, newval: str) -> int:
            return self._run(self._aio.set_command(newval))

    @classmethod
    def FindWeighScale(cls, func: str) -> YWeighScale:
        """
        Retrieves a weighing scale sensor for a given identifier.
        The identifier can be specified using several formats:

        - FunctionLogicalName
        - ModuleSerialNumber.FunctionIdentifier
        - ModuleSerialNumber.FunctionLogicalName
        - ModuleLogicalName.FunctionIdentifier
        - ModuleLogicalName.FunctionLogicalName


        This function does not require that the weighing scale sensor is online at the time
        it is invoked. The returned object is nevertheless valid.
        Use the method YWeighScale.isOnline() to test if the weighing scale sensor is
        indeed online at a given time. In case of ambiguity when looking for
        a weighing scale sensor by logical name, no error is notified: the first instance
        found is returned. The search is performed first by hardware name,
        then by logical name.

        If a call to this object's is_online() method returns FALSE although
        you are certain that the matching device is plugged, make sure that you did
        call registerHub() at application initialization time.

        @param func : a string that uniquely characterizes the weighing scale sensor, for instance
                YWBRIDG1.weighScale1.

        @return a YWeighScale object allowing you to drive the weighing scale sensor.
        """
        return cls._proxy(cls, YWeighScale_aio.FindWeighScale(func))

    @classmethod
    def FindWeighScaleInContext(cls, yctx: YAPIContext, func: str) -> YWeighScale:
        """
        Retrieves a weighing scale sensor for a given identifier in a YAPI context.
        The identifier can be specified using several formats:

        - FunctionLogicalName
        - ModuleSerialNumber.FunctionIdentifier
        - ModuleSerialNumber.FunctionLogicalName
        - ModuleLogicalName.FunctionIdentifier
        - ModuleLogicalName.FunctionLogicalName


        This function does not require that the weighing scale sensor is online at the time
        it is invoked. The returned object is nevertheless valid.
        Use the method YWeighScale.isOnline() to test if the weighing scale sensor is
        indeed online at a given time. In case of ambiguity when looking for
        a weighing scale sensor by logical name, no error is notified: the first instance
        found is returned. The search is performed first by hardware name,
        then by logical name.

        @param yctx : a YAPI context
        @param func : a string that uniquely characterizes the weighing scale sensor, for instance
                YWBRIDG1.weighScale1.

        @return a YWeighScale object allowing you to drive the weighing scale sensor.
        """
        return cls._proxy(cls, YWeighScale_aio.FindWeighScaleInContext(yctx, func))

    if not _IS_MICROPYTHON:
        def registerValueCallback(self, callback: YWeighScaleValueCallback) -> int:
            """
            Registers the callback function that is invoked on every change of advertised value.
            The callback is invoked only during the execution of ySleep or yHandleEvents.
            This provides control over the time when the callback is triggered. For good responsiveness, remember to call
            one of these two functions periodically. To unregister a callback, pass a None pointer as argument.

            @param callback : the callback function to call, or a None pointer. The callback function should take two
                    arguments: the function object of which the value has changed, and the character string describing
                    the new advertised value.
            @noreturn
            """
            return super().registerValueCallback(callback)

    if not _IS_MICROPYTHON:
        def registerTimedReportCallback(self, callback: YWeighScaleTimedReportCallback) -> int:
            """
            Registers the callback function that is invoked on every periodic timed notification.
            The callback is invoked only during the execution of ySleep or yHandleEvents.
            This provides control over the time when the callback is triggered. For good responsiveness, remember to call
            one of these two functions periodically. To unregister a callback, pass a None pointer as argument.

            @param callback : the callback function to call, or a None pointer. The callback function should take two
                    arguments: the function object of which the value has changed, and an YMeasure object describing
                    the new advertised value.
            @noreturn
            """
            return super().registerTimedReportCallback(callback)

    if not _DYNAMIC_HELPERS:
        def tare(self) -> int:
            """
            Adapts the load cell signal bias (stored in the corresponding genericSensor)
            so that the current signal corresponds to a zero weight. Remember to call the
            saveToFlash() method of the module if the modification must be kept.

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.tare())

    if not _DYNAMIC_HELPERS:
        def setupSpan(self, currWeight: float, maxWeight: float) -> int:
            """
            Configures the load cell span parameters (stored in the corresponding genericSensor)
            so that the current signal corresponds to the specified reference weight.

            @param currWeight : reference weight presently on the load cell.
            @param maxWeight : maximum weight to be expected on the load cell.

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.setupSpan(currWeight, maxWeight))

    if not _DYNAMIC_HELPERS:
        def set_offsetAvgCompensationTable(self, tempValues: list[float], compValues: list[float]) -> int:
            """
            Records a weight offset thermal compensation table, in order to automatically correct the
            measured weight based on the averaged compensation temperature.
            The weight correction will be applied by linear interpolation between specified points.

            @param tempValues : array of floating point numbers, corresponding to all averaged
                    temperatures for which an offset correction is specified.
            @param compValues : array of floating point numbers, corresponding to the offset correction
                    to apply for each of the temperature included in the first
                    argument, index by index.

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_offsetAvgCompensationTable(tempValues, compValues))

    if not _DYNAMIC_HELPERS:
        def loadOffsetAvgCompensationTable(self, tempValues: list[float], compValues: list[float]) -> int:
            """
            Retrieves the weight offset thermal compensation table previously configured using the
            set_offsetAvgCompensationTable function.
            The weight correction is applied by linear interpolation between specified points.

            @param tempValues : array of floating point numbers, that is filled by the function
                    with all averaged temperatures for which an offset correction is specified.
            @param compValues : array of floating point numbers, that is filled by the function
                    with the offset correction applied for each of the temperature
                    included in the first argument, index by index.

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.loadOffsetAvgCompensationTable(tempValues, compValues))

    if not _DYNAMIC_HELPERS:
        def set_offsetChgCompensationTable(self, tempValues: list[float], compValues: list[float]) -> int:
            """
            Records a weight offset thermal compensation table, in order to automatically correct the
            measured weight based on the variation of temperature.
            The weight correction will be applied by linear interpolation between specified points.

            @param tempValues : array of floating point numbers, corresponding to temperature
                    variations for which an offset correction is specified.
            @param compValues : array of floating point numbers, corresponding to the offset correction
                    to apply for each of the temperature variation included in the first
                    argument, index by index.

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_offsetChgCompensationTable(tempValues, compValues))

    if not _DYNAMIC_HELPERS:
        def loadOffsetChgCompensationTable(self, tempValues: list[float], compValues: list[float]) -> int:
            """
            Retrieves the weight offset thermal compensation table previously configured using the
            set_offsetChgCompensationTable function.
            The weight correction is applied by linear interpolation between specified points.

            @param tempValues : array of floating point numbers, that is filled by the function
                    with all temperature variations for which an offset correction is specified.
            @param compValues : array of floating point numbers, that is filled by the function
                    with the offset correction applied for each of the temperature
                    variation included in the first argument, index by index.

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.loadOffsetChgCompensationTable(tempValues, compValues))

    if not _DYNAMIC_HELPERS:
        def set_spanAvgCompensationTable(self, tempValues: list[float], compValues: list[float]) -> int:
            """
            Records a weight span thermal compensation table, in order to automatically correct the
            measured weight based on the compensation temperature.
            The weight correction will be applied by linear interpolation between specified points.

            @param tempValues : array of floating point numbers, corresponding to all averaged
                    temperatures for which a span correction is specified.
            @param compValues : array of floating point numbers, corresponding to the span correction
                    (in percents) to apply for each of the temperature included in the first
                    argument, index by index.

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_spanAvgCompensationTable(tempValues, compValues))

    if not _DYNAMIC_HELPERS:
        def loadSpanAvgCompensationTable(self, tempValues: list[float], compValues: list[float]) -> int:
            """
            Retrieves the weight span thermal compensation table previously configured using the
            set_spanAvgCompensationTable function.
            The weight correction is applied by linear interpolation between specified points.

            @param tempValues : array of floating point numbers, that is filled by the function
                    with all averaged temperatures for which an span correction is specified.
            @param compValues : array of floating point numbers, that is filled by the function
                    with the span correction applied for each of the temperature
                    included in the first argument, index by index.

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.loadSpanAvgCompensationTable(tempValues, compValues))

    if not _DYNAMIC_HELPERS:
        def set_spanChgCompensationTable(self, tempValues: list[float], compValues: list[float]) -> int:
            """
            Records a weight span thermal compensation table, in order to automatically correct the
            measured weight based on the variation of temperature.
            The weight correction will be applied by linear interpolation between specified points.

            @param tempValues : array of floating point numbers, corresponding to all variations of
                    temperatures for which a span correction is specified.
            @param compValues : array of floating point numbers, corresponding to the span correction
                    (in percents) to apply for each of the temperature variation included
                    in the first argument, index by index.

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_spanChgCompensationTable(tempValues, compValues))

    if not _DYNAMIC_HELPERS:
        def loadSpanChgCompensationTable(self, tempValues: list[float], compValues: list[float]) -> int:
            """
            Retrieves the weight span thermal compensation table previously configured using the
            set_spanChgCompensationTable function.
            The weight correction is applied by linear interpolation between specified points.

            @param tempValues : array of floating point numbers, that is filled by the function
                    with all variation of temperature for which an span correction is specified.
            @param compValues : array of floating point numbers, that is filled by the function
                    with the span correction applied for each of variation of temperature
                    included in the first argument, index by index.

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.loadSpanChgCompensationTable(tempValues, compValues))

    # --- (end of YWeighScale implementation)




# -*- coding: utf-8 -*-
# ********************************************************************
#
#  $Id: svn_id $
#
#  Asyncio implementation of YLed
#
#  - - - - - - - - - License information: - - - - - - - - -
#
#  Copyright (C) 2011 and beyond by Yoctopuce Sarl, Switzerland.
#
#  Yoctopuce Sarl (hereafter Licensor) grants to you a perpetual
#  non-exclusive license to use, modify, copy and integrate this
#  file into your software for the sole purpose of interfacing
#  with Yoctopuce products.
#
#  You may reproduce and distribute copies of this file in
#  source or object form, as long as the sole purpose of this
#  code is to interface with Yoctopuce products. You must retain
#  this notice in the distributed source file.
#
#  You should refer to Yoctopuce General Terms and Conditions
#  for additional information regarding your rights and
#  obligations.
#
#  THE SOFTWARE AND DOCUMENTATION ARE PROVIDED 'AS IS' WITHOUT
#  WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING
#  WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, FITNESS
#  FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO
#  EVENT SHALL LICENSOR BE LIABLE FOR ANY INCIDENTAL, SPECIAL,
#  INDIRECT OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA,
#  COST OF PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR
#  SERVICES, ANY CLAIMS BY THIRD PARTIES (INCLUDING BUT NOT
#  LIMITED TO ANY DEFENSE THEREOF), ANY CLAIMS FOR INDEMNITY OR
#  CONTRIBUTION, OR OTHER SIMILAR COSTS, WHETHER ASSERTED ON THE
#  BASIS OF CONTRACT, TORT (INCLUDING NEGLIGENCE), BREACH OF
#  WARRANTY, OR OTHERWISE.
#
# *********************************************************************
"""
Yoctopuce library: Asyncio implementation of YLed
version: PATCH_WITH_VERSION
requires: yocto_api_aio
"""
from __future__ import annotations

import sys

# On MicroPython, code below will be wiped out at compile time
if sys.implementation.name != "micropython":
    # In CPython, enable edit-time type checking, including Final declaration
    from typing import Any, Union, Final
    from collections.abc import Callable, Awaitable
    from .yocto_api_aio import const, _IS_MICROPYTHON
else:
    # In our micropython VM, common generic types are global built-ins
    # Others such as TypeVar should be avoided when using micropython,
    # as they produce overhead in runtime code
    # Final is translated into const() expressions before compilation
    _IS_MICROPYTHON: Final[bool] = True # noqa

from .yocto_api_aio import (
    YAPIContext, YAPI, YAPI_Exception, HwId, hwid2str, YFunction
)

# --- (YLed class start)
if not _IS_MICROPYTHON:
    # For CPython, use strongly typed callback types
    try:
        YLedValueCallback = Union[Callable[['YLed', str], Awaitable[None]], None]
    except TypeError:
        YLedValueCallback = Union[Callable, Awaitable]

# noinspection PyProtectedMember
class YLed(YFunction):
    """
    The YLed class allows you to drive a monocolor LED.
    You can not only to drive the intensity of the LED, but also to
    have it blink at various preset frequencies.

    """
    # --- (end of YLed class start)
    if not _IS_MICROPYTHON:
        # --- (YLed return codes)
        LUMINOSITY_INVALID: Final[int] = YAPI.INVALID_UINT
        POWER_OFF: Final[int] = 0
        POWER_ON: Final[int] = 1
        POWER_INVALID: Final[int] = -1
        BLINKING_STILL: Final[int] = 0
        BLINKING_RELAX: Final[int] = 1
        BLINKING_AWARE: Final[int] = 2
        BLINKING_RUN: Final[int] = 3
        BLINKING_CALL: Final[int] = 4
        BLINKING_PANIC: Final[int] = 5
        BLINKING_INVALID: Final[int] = -1
        # --- (end of YLed return codes)

    # --- (YLed attributes declaration)
    _power: int
    _luminosity: int
    _blinking: int
    _valueCallback: YLedValueCallback
    # --- (end of YLed attributes declaration)


    def __init__(self, yctx: YAPIContext, func: str):
        super().__init__(yctx, func)
        self._className = 'Led'
        # --- (YLed constructor)
        self._power = YLed.POWER_INVALID
        self._luminosity = YLed.LUMINOSITY_INVALID
        self._blinking = YLed.BLINKING_INVALID
        # --- (end of YLed constructor)

    # --- (YLed implementation)

    @staticmethod
    def FirstLed() -> Union[YLed, None]:
        """
        Starts the enumeration of monochrome LEDs currently accessible.
        Use the method YLed.nextLed() to iterate on
        next monochrome LEDs.

        @return a pointer to a YLed object, corresponding to
                the first monochrome LED currently online, or a None pointer
                if there are none.
        """
        next_hwid: Union[HwId, None] = YAPI._yHash.getFirstHardwareId('Led')
        if not next_hwid:
            return None
        return YLed.FindLed(hwid2str(next_hwid))

    @staticmethod
    def FirstLedInContext(yctx: YAPIContext) -> Union[YLed, None]:
        """
        Starts the enumeration of monochrome LEDs currently accessible.
        Use the method YLed.nextLed() to iterate on
        next monochrome LEDs.

        @param yctx : a YAPI context.

        @return a pointer to a YLed object, corresponding to
                the first monochrome LED currently online, or a None pointer
                if there are none.
        """
        next_hwid: Union[HwId, None] = yctx._yHash.getFirstHardwareId('Led')
        if not next_hwid:
            return None
        return YLed.FindLedInContext(yctx, hwid2str(next_hwid))

    def nextLed(self):
        """
        Continues the enumeration of monochrome LEDs started using yFirstLed().
        Caution: You can't make any assumption about the returned monochrome LEDs order.
        If you want to find a specific a monochrome LED, use Led.findLed()
        and a hardwareID or a logical name.

        @return a pointer to a YLed object, corresponding to
                a monochrome LED currently online, or a None pointer
                if there are no more monochrome LEDs to enumerate.
        """
        next_hwid: Union[HwId, None] = None
        try:
            hwid: HwId = self._yapi._yHash.resolveHwID(self._className, self._func)
            next_hwid = self._yapi._yHash.getNextHardwareId(self._className, hwid)
        except YAPI_Exception:
            pass
        if not next_hwid:
            return None
        return YLed.FindLedInContext(self._yapi, hwid2str(next_hwid))

    def _parseAttr(self, json_val: dict) -> None:
        if 'power' in json_val:
            self._power = json_val["power"] > 0
        if 'luminosity' in json_val:
            self._luminosity = json_val["luminosity"]
        if 'blinking' in json_val:
            self._blinking = json_val["blinking"]
        super()._parseAttr(json_val)

    async def get_power(self) -> int:
        """
        Returns the current LED state.

        @return either YLed.POWER_OFF or YLed.POWER_ON, according to the current LED state

        On failure, throws an exception or returns YLed.POWER_INVALID.
        """
        res: int
        if self._cacheExpiration <= YAPI.GetTickCount():
            if await self.load(self._yapi.GetCacheValidity()) != YAPI.SUCCESS:
                return YLed.POWER_INVALID
        res = self._power
        return res

    async def set_power(self, newval: int) -> int:
        """
        Changes the state of the LED.

        @param newval : either YLed.POWER_OFF or YLed.POWER_ON, according to the state of the LED

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        rest_val = "1" if newval > 0 else "0"
        return await self._setAttr("power", rest_val)

    async def get_luminosity(self) -> int:
        """
        Returns the current LED intensity (in per cent).

        @return an integer corresponding to the current LED intensity (in per cent)

        On failure, throws an exception or returns YLed.LUMINOSITY_INVALID.
        """
        res: int
        if self._cacheExpiration <= YAPI.GetTickCount():
            if await self.load(self._yapi.GetCacheValidity()) != YAPI.SUCCESS:
                return YLed.LUMINOSITY_INVALID
        res = self._luminosity
        return res

    async def set_luminosity(self, newval: int) -> int:
        """
        Changes the current LED intensity (in per cent). Remember to call the
        saveToFlash() method of the module if the modification must be kept.

        @param newval : an integer corresponding to the current LED intensity (in per cent)

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        rest_val = str(newval)
        return await self._setAttr("luminosity", rest_val)

    async def get_blinking(self) -> int:
        """
        Returns the current LED signaling mode.

        @return a value among YLed.BLINKING_STILL, YLed.BLINKING_RELAX, YLed.BLINKING_AWARE,
        YLed.BLINKING_RUN, YLed.BLINKING_CALL and YLed.BLINKING_PANIC corresponding to the current LED signaling mode

        On failure, throws an exception or returns YLed.BLINKING_INVALID.
        """
        res: int
        if self._cacheExpiration <= YAPI.GetTickCount():
            if await self.load(self._yapi.GetCacheValidity()) != YAPI.SUCCESS:
                return YLed.BLINKING_INVALID
        res = self._blinking
        return res

    async def set_blinking(self, newval: int) -> int:
        """
        Changes the current LED signaling mode.

        @param newval : a value among YLed.BLINKING_STILL, YLed.BLINKING_RELAX, YLed.BLINKING_AWARE,
        YLed.BLINKING_RUN, YLed.BLINKING_CALL and YLed.BLINKING_PANIC corresponding to the current LED signaling mode

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        rest_val = str(newval)
        return await self._setAttr("blinking", rest_val)

    @staticmethod
    def FindLed(func: str) -> YLed:
        """
        Retrieves a monochrome LED for a given identifier.
        The identifier can be specified using several formats:

        - FunctionLogicalName
        - ModuleSerialNumber.FunctionIdentifier
        - ModuleSerialNumber.FunctionLogicalName
        - ModuleLogicalName.FunctionIdentifier
        - ModuleLogicalName.FunctionLogicalName


        This function does not require that the monochrome LED is online at the time
        it is invoked. The returned object is nevertheless valid.
        Use the method YLed.isOnline() to test if the monochrome LED is
        indeed online at a given time. In case of ambiguity when looking for
        a monochrome LED by logical name, no error is notified: the first instance
        found is returned. The search is performed first by hardware name,
        then by logical name.

        If a call to this object's is_online() method returns FALSE although
        you are certain that the matching device is plugged, make sure that you did
        call registerHub() at application initialization time.

        @param func : a string that uniquely characterizes the monochrome LED, for instance
                YBUZZER2.led1.

        @return a YLed object allowing you to drive the monochrome LED.
        """
        obj: Union[YLed, None]
        obj = YFunction._FindFromCache("Led", func)
        if obj is None:
            obj = YLed(YAPI, func)
            YFunction._AddToCache("Led", func, obj)
        return obj

    @staticmethod
    def FindLedInContext(yctx: YAPIContext, func: str) -> YLed:
        """
        Retrieves a monochrome LED for a given identifier in a YAPI context.
        The identifier can be specified using several formats:

        - FunctionLogicalName
        - ModuleSerialNumber.FunctionIdentifier
        - ModuleSerialNumber.FunctionLogicalName
        - ModuleLogicalName.FunctionIdentifier
        - ModuleLogicalName.FunctionLogicalName


        This function does not require that the monochrome LED is online at the time
        it is invoked. The returned object is nevertheless valid.
        Use the method YLed.isOnline() to test if the monochrome LED is
        indeed online at a given time. In case of ambiguity when looking for
        a monochrome LED by logical name, no error is notified: the first instance
        found is returned. The search is performed first by hardware name,
        then by logical name.

        @param yctx : a YAPI context
        @param func : a string that uniquely characterizes the monochrome LED, for instance
                YBUZZER2.led1.

        @return a YLed object allowing you to drive the monochrome LED.
        """
        obj: Union[YLed, None]
        obj = YFunction._FindFromCacheInContext(yctx, "Led", func)
        if obj is None:
            obj = YLed(yctx, func)
            YFunction._AddToCache("Led", func, obj)
        return obj

    if not _IS_MICROPYTHON:
        async def registerValueCallback(self, callback: YLedValueCallback) -> int:
            """
            Registers the callback function that is invoked on every change of advertised value.
            The callback is invoked only during the execution of ySleep or yHandleEvents.
            This provides control over the time when the callback is triggered. For good responsiveness, remember to call
            one of these two functions periodically. To unregister a callback, pass a None pointer as argument.

            @param callback : the callback function to call, or a None pointer. The callback function should take two
                    arguments: the function object of which the value has changed, and the character string describing
                    the new advertised value.
            @noreturn
            """
            return await super().registerValueCallback(callback)

    # --- (end of YLed implementation)




#!/usr/bin/python
# -*- coding: utf-8 -*-0
import sys
import os
import time

# add ../../Sources to the PYTHONPATH
sys.path.append(os.path.join("..", "..", "Sources"))
from yocto_api import *


def main():
    # Setup the API to use local USB devices
    errmsg = YRefParam()
    if YAPI.RegisterHub("usb", errmsg) != YAPI.SUCCESS:
        sys.exit("init error" + errmsg.value)

    # Enumerate all connected sensors
    sensorList = []
    sensor = YSensor.FirstSensor()
    while sensor is not None:
        sensorList.append(sensor)
        sensor = sensor.nextSensor()
    if len(sensorList) == 0:
        sys.exit("No Yoctopuce sensor connected (check USB cable)")

    # Generate consolidated CSV output for all sensors
    data = YConsolidatedDataSet(0, 0, sensorList)
    record = []
    while data.nextRecord(record) < 100:
        line = datetime.datetime.fromtimestamp(record[0]).isoformat()
        for idx in range(1, len(record)):
            line += ";%.3f" % record[idx]
        print(line)
    YAPI.FreeAPI()


if __name__ == '__main__':
    main()
